#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import hashlib
import logging
import os
import uuid
from pathlib import PurePosixPath

from django.conf import settings
from django.contrib.auth.models import AbstractUser, AnonymousUser
from django.db import models
from django_celery_beat.models import PeriodicTask

from django_base_ai import dispatch
from django_base_ai.system.ext_field.core_base_auth import CoreBaseAuthField
from django_base_ai.system.ext_field.models_safe import EncrypyField
from django_base_ai.utils.models import CoreModel, newuuid
table_prefix = "base_ai_"
logger = logging.getLogger(__name__)


def media_file_name(instance, filename):
    h = instance.md5sum
    basename, ext = os.path.splitext(filename)
    return PurePosixPath("files", h + ext.lower())


# 应用管理 appid
def create_agent_id():
    return f"appid_{str(uuid.uuid4().hex)}"


# 应用管理 密钥
def create_secret():
    return f"secret_{uuid.uuid1().hex + uuid.uuid4().hex}"


class Users(CoreModel, AbstractUser):
    dept = models.ForeignKey(
        to="Dept",
        verbose_name="所属部门",
        on_delete=models.PROTECT,
        db_constraint=False,
        null=True,
        blank=True,
        help_text="关联部门",
        db_index=True,
    )
    parent = models.ForeignKey(
        to="Users",
        on_delete=models.CASCADE,
        default=None,
        verbose_name="上级领导",
        db_constraint=False,
        null=True,
        blank=True,
        help_text="上级领导",
        db_index=True,
    )
    username = models.CharField(
        max_length=255,
        unique=True,
        db_index=True,
        verbose_name="用户账号",
        help_text="用户账号",
        null=False,
        blank=False,
    )
    employee_no = models.CharField(
        max_length=255, unique=True, db_index=True, null=True, blank=True, verbose_name="工号", help_text="工号"
    )
    position = models.CharField(max_length=255, verbose_name="职位", null=True, blank=True, help_text="职位")
    email = models.EmailField(
        max_length=255, verbose_name="邮箱", null=True, blank=True, help_text="邮箱", db_index=True
    )
    mobile = EncrypyField(max_length=255, verbose_name="手机", null=True, blank=True, help_text="手机")
    landline = models.CharField(max_length=255, verbose_name="座机", null=True, blank=True, help_text="固定电话")
    avatar = models.CharField(max_length=255, verbose_name="头像", null=True, blank=True, help_text="头像")
    name = models.CharField(
        max_length=255, verbose_name="姓名", help_text="姓名", db_index=True, null=False, blank=False, default=""
    )
    alias = models.CharField(
        max_length=255, verbose_name="成员别名", help_text="成员别名", db_index=True, null=True, blank=True
    )
    main_department = models.CharField(max_length=255, verbose_name="主部门", help_text="主部门", null=True, blank=True)
    sort = models.IntegerField(default=1, verbose_name="排序", help_text="排序")
    pwd_defaulted = models.BooleanField(default=True, verbose_name="密码是否为默认", help_text="密码是否为默认")
    GENDER_CHOICES = (
        (0, "保密"),
        (1, "男"),
        (2, "女"),
    )
    gender = models.IntegerField(
        choices=GENDER_CHOICES, default=0, verbose_name="性别", null=True, blank=True, help_text="性别"
    )
    USER_TYPE = (
        (0, "正式"),
        (1, "借调"),
        (2, "外包"),
    )
    user_type = models.IntegerField(
        choices=USER_TYPE,
        default=1,
        verbose_name="用户类型",
        null=True,
        blank=True,
        help_text="用户类型",
        db_index=True,
    )

    post = models.ManyToManyField(
        to="Post", blank=True, verbose_name="关联岗位", db_constraint=False, help_text="关联岗位"
    )
    role = models.ManyToManyField(
        to="Role", blank=True, verbose_name="关联角色", db_constraint=False, help_text="关联角色"
    )

    last_token = models.CharField(
        max_length=255, null=True, blank=True, verbose_name="最后一次登录Token", help_text="最后一次登录Token"
    )
    skin = models.TextField(verbose_name="皮肤", choices=(), null=True, blank=True)
    node_type = models.IntegerField(
        default=4, verbose_name="节点类型", help_text="用于处理组织架构标签4表示用户", db_index=True
    )
    user_config = models.JSONField(
        max_length=50000, verbose_name="用户配置", help_text="用户配置", null=True, blank=True, default=dict
    )
    authorization_code = EncrypyField(max_length=255, verbose_name="授权码", null=True, blank=True, help_text="授权码")

    # 禁止落库的用户名，遇到则直接跳过 save，不新增也不更新
    _SKIP_SAVE_USERNAMES = ("AnonymousUser", "_ANONYMOUS_USER_BLOCKED_")

    def set_password(self, raw_password):
        super().set_password(raw_password)

    def save(self, *args, **kwargs):
        if self.username in self._SKIP_SAVE_USERNAMES:
            return
        super().save(*args, **kwargs)

    def clean(self):
        """模型验证：禁止创建用户名为 AnonymousUser 的用户"""
        from django.core.exceptions import ValidationError

        if self.username == "AnonymousUser":
            raise ValidationError("不能创建用户名为 'AnonymousUser' 的用户")
        super().clean()

    class Meta:
        db_table = table_prefix + "system_users"
        verbose_name = verbose_name_plural = "用户表"
        ordering = ("-create_datetime",)


class MySuperiors(CoreModel):
    user_info = models.ForeignKey(
        to="Users",
        db_constraint=False,
        related_name="userSuperiors",
        on_delete=models.PROTECT,
        verbose_name="协作人",
        help_text="协作人",
    )

    class Meta:
        db_table = table_prefix + "my_superiors"
        verbose_name = verbose_name_plural = "协作人"
        ordering = ("-create_datetime",)


class Post(CoreModel):
    name = models.CharField(null=False, max_length=64, verbose_name="岗位名称", help_text="岗位名称", db_index=True)
    code = models.CharField(max_length=32, verbose_name="岗位编码", help_text="岗位编码", db_index=True)
    sort = models.IntegerField(default=1, verbose_name="岗位顺序", help_text="岗位顺序")
    STATUS_CHOICES = (
        (0, "离职"),
        (1, "在职"),
    )
    status = models.IntegerField(
        choices=STATUS_CHOICES, default=1, verbose_name="岗位状态", help_text="岗位状态", db_index=True
    )

    class Meta:
        db_table = table_prefix + "system_post"
        verbose_name = verbose_name_plural = "岗位表"
        ordering = ("sort",)


class Role(CoreModel):
    name = models.CharField(max_length=64, verbose_name="角色名称", help_text="角色名称", db_index=True)
    key = models.CharField(max_length=64, unique=True, verbose_name="权限字符", help_text="权限字符")
    sort = models.IntegerField(default=1, verbose_name="角色顺序", help_text="角色顺序")
    status = models.BooleanField(default=True, verbose_name="角色状态", help_text="角色状态", db_index=True)
    admin = models.BooleanField(default=False, verbose_name="是否为admin", help_text="是否为admin", db_index=True)
    DATASCOPE_CHOICES = (
        (0, "仅本人数据权限"),
        (1, "本部门及以下数据权限"),
        (2, "本部门数据权限"),
        (3, "全部数据权限"),
        (4, "自定数据权限"),
    )
    data_range = models.IntegerField(
        default=0, choices=DATASCOPE_CHOICES, verbose_name="数据权限范围", help_text="数据权限范围"
    )
    dept = models.ManyToManyField(
        to="Dept", verbose_name="数据权限-关联部门", db_constraint=False, help_text="数据权限-关联部门"
    )
    menu = models.ManyToManyField(to="Menu", verbose_name="关联菜单", db_constraint=False, help_text="关联菜单")
    permission = models.ManyToManyField(
        to="MenuButton", verbose_name="关联菜单的接口按钮", db_constraint=False, help_text="关联菜单的接口按钮"
    )
    node_type = models.IntegerField(default=2, verbose_name="节点类型", help_text="用于处理组织架构菜单2表示角色")
    remark = models.TextField(verbose_name="备注", help_text="备注", null=True, blank=True)

    class Meta:
        db_table = table_prefix + "system_role"
        verbose_name = verbose_name_plural = "角色表"
        ordering = ("sort",)


class Dept(CoreModel):
    parent = models.ForeignKey(
        to="Dept",
        on_delete=models.CASCADE,
        default=None,
        verbose_name="上级部门",
        db_constraint=False,
        null=True,
        blank=True,
        help_text="上级部门",
        db_index=True,
    )
    name = models.CharField(max_length=255, verbose_name="部门名称", help_text="部门名称", db_index=True)
    short_name = models.CharField(
        max_length=255, verbose_name="部门简称", help_text="部门简称", null=True, blank=True, default=""
    )
    dept_full_path = models.CharField(
        max_length=255, verbose_name="架构全路径", help_text="架构全路径", null=True, blank=True, default=""
    )
    key = models.CharField(
        max_length=255,
        unique=True,
        null=True,
        blank=True,
        verbose_name="关联字符",
        help_text="关联字符",
        default=newuuid,
    )
    sort = models.IntegerField(default=1, verbose_name="显示排序", help_text="显示排序")
    owner_user = models.ForeignKey(
        to=Users,
        related_name="owner_user",
        null=True,
        blank=True,
        verbose_name="关联用户",
        db_constraint=False,
        default=None,
        on_delete=models.CASCADE,
        help_text="关联用户",
    )
    phone = EncrypyField(max_length=255, verbose_name="座机", null=True, blank=True, help_text="座机")
    email = models.EmailField(
        max_length=255, verbose_name="邮箱", null=True, blank=True, help_text="邮箱", db_index=True
    )
    status = models.BooleanField(
        default=True, verbose_name="部门状态", null=True, blank=True, help_text="部门状态", db_index=True
    )
    node_type = models.IntegerField(default=1, verbose_name="节点类型", help_text="用于处理组织架构菜单1表示部门")

    @classmethod
    def recursion_dept_info(cls, dept_id: int, dept_all_list=None, dept_list=None):
        """
        递归获取部门的所有下级部门
        :param dept_id: 需要获取的id
        :param dept_all_list: 所有列表
        :param dept_list: 递归list
        :return:
        """
        if not dept_all_list:
            dept_all_list = Dept.objects.values("id", "parent")
        if dept_list is None:
            dept_list = [dept_id]
        for ele in dept_all_list:
            if ele.get("parent") == dept_id:
                dept_list.append(ele.get("id"))
                cls.recursion_dept_info(ele.get("id"), dept_all_list, dept_list)
        return list(set(dept_list))

    class Meta:
        db_table = table_prefix + "system_dept"
        verbose_name = verbose_name_plural = "部门表"
        ordering = ("sort",)


# 用户自定义标签
class Tags(CoreModel):
    name = models.CharField(max_length=255, verbose_name="标签名称", help_text="标签名称", db_index=True)
    sort = models.IntegerField(default=1, verbose_name="标签顺序", help_text="标签顺序")
    status = models.BooleanField(default=True, verbose_name="标签状态", help_text="标签状态", db_index=True)
    dept = models.ManyToManyField(
        to="Dept", blank=True, verbose_name="关联部门", db_constraint=False, help_text="关联部门"
    )
    user_info = models.ManyToManyField(
        to="Users",
        blank=True,
        verbose_name="关联用户",
        db_constraint=False,
        help_text="关联用户",
        related_name="tags_user_info",
    )
    read_user_info = models.ManyToManyField(
        to="Users",
        blank=True,
        verbose_name="仅使用",
        db_constraint=False,
        help_text="仅使用",
        related_name="tags_read_user_info",
    )
    edit_user_info = models.ManyToManyField(
        to="Users",
        blank=True,
        verbose_name="读写",
        db_constraint=False,
        help_text="读写",
        related_name="tags_edit_user_info",
    )
    node_type = models.IntegerField(default=3, verbose_name="节点类型", help_text="用于处理组织架构标签3表示标签")

    remark = models.TextField(verbose_name="备注", help_text="备注", null=True, blank=True)

    class Meta:
        db_table = table_prefix + "tags"
        verbose_name = verbose_name_plural = "自定义标签"
        ordering = ("-create_datetime",)


# 组织架构基础类
class CoreBaseAuth(models.Model):
    target_dept = models.ManyToManyField(
        to=Dept, blank=True, db_constraint=False, verbose_name="部门", help_text="部门", related_name="target_dept"
    )
    target_role = models.ManyToManyField(
        to=Role, blank=True, db_constraint=False, verbose_name="角色", help_text="角色", related_name="target_role"
    )
    target_tags = models.ManyToManyField(
        to=Tags, blank=True, db_constraint=False, verbose_name="标签", help_text="标签", related_name="target_tags"
    )
    target_users = models.ManyToManyField(
        to=Users, blank=True, db_constraint=False, verbose_name="用户", help_text="用户", related_name="target_users"
    )
    target_data = models.JSONField(
        max_length=50000, verbose_name="上送json", help_text="上送json", null=True, blank=True, default=dict
    )

    class Meta:
        db_table = table_prefix + "auth"
        verbose_name = verbose_name_plural = "基础授权表"
        ordering = ("-id",)


class Menu(CoreModel):
    parent = models.ForeignKey(
        to="Menu",
        on_delete=models.CASCADE,
        verbose_name="上级菜单",
        null=True,
        blank=True,
        db_constraint=False,
        help_text="上级菜单",
    )
    name = models.CharField(max_length=64, verbose_name="菜单名称", help_text="菜单名称", db_index=True)
    path = models.CharField(
        max_length=128, verbose_name="路由地址", null=True, blank=True, help_text="路由地址", db_index=True
    )
    component = models.CharField(max_length=128, verbose_name="组件地址", null=True, blank=True, help_text="组件地址")
    component_name = models.CharField(
        max_length=50, verbose_name="组件名称", null=True, blank=True, help_text="组件名称"
    )
    redirect = models.CharField(
        max_length=128, verbose_name="重定向到子路由", null=True, blank=True, help_text="重定向到子路由"
    )
    sort = models.IntegerField(default=1, verbose_name="显示排序", null=True, blank=True, help_text="显示排序")
    status = models.BooleanField(default=True, blank=True, verbose_name="菜单状态", help_text="菜单状态", db_index=True)
    meta = models.JSONField(max_length=5000, verbose_name="meta", help_text="meta", null=True, blank=True)

    class Meta:
        db_table = table_prefix + "system_menu"
        verbose_name = verbose_name_plural = "菜单表"
        ordering = ("sort",)


class MenuButton(CoreModel):
    menu = models.ForeignKey(
        to="Menu",
        db_constraint=False,
        related_name="menuPermission",
        on_delete=models.PROTECT,
        verbose_name="关联菜单",
        help_text="关联菜单",
    )
    name = models.CharField(max_length=64, verbose_name="名称", help_text="名称")
    value = models.CharField(max_length=64, verbose_name="权限值", help_text="权限值", db_index=True)
    api = models.CharField(max_length=200, verbose_name="接口地址", help_text="接口地址", db_index=True)
    METHOD_CHOICES = (
        (0, "GET"),
        (1, "POST"),
        (2, "PUT"),
        (3, "DELETE"),
    )
    method = models.IntegerField(
        default=0, verbose_name="接口请求方法", null=True, blank=True, help_text="接口请求方法"
    )

    class Meta:
        db_table = table_prefix + "system_menu_button"
        verbose_name = verbose_name_plural = "菜单权限表"
        ordering = ("-name",)


class Dictionary(CoreModel):
    TYPE_LIST = (
        (0, "text"),
        (1, "number"),
        (2, "date"),
        (3, "datetime"),
        (4, "time"),
        (5, "files"),
        (6, "boolean"),
        (7, "images"),
        (8, "datetimerange"),
        (9, "tree"),
    )
    label = models.CharField(
        max_length=100, blank=True, null=True, verbose_name="字典名称", help_text="字典名称", db_index=True
    )
    value = models.CharField(
        max_length=200, blank=True, null=True, verbose_name="字典编号", help_text="字典编号/实际值", db_index=True
    )
    parent = models.ForeignKey(
        to="self",
        related_name="sublist",
        db_constraint=False,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        verbose_name="父级",
        help_text="父级",
    )
    type = models.IntegerField(choices=TYPE_LIST, default=0, verbose_name="数据值类型", help_text="数据值类型")
    color = models.CharField(max_length=225, blank=True, null=True, verbose_name="颜色", help_text="颜色")
    icon = models.CharField(max_length=225, blank=True, null=True, verbose_name="图标", help_text="图标")
    is_value = models.BooleanField(
        default=False, verbose_name="是否为value值", help_text="是否为value值,用来做具体值存放"
    )
    status = models.BooleanField(default=True, verbose_name="状态", help_text="状态", db_index=True)
    sort = models.IntegerField(default=1, verbose_name="显示排序", null=True, blank=True, help_text="显示排序")
    remark = models.CharField(max_length=2000, blank=True, null=True, verbose_name="备注", help_text="备注")

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        super().save(force_insert, force_update, using, update_fields)
        dispatch.refresh_dictionary()  # 有更新则刷新字典配置

    def delete(self, using=None, keep_parents=False):
        res = super().delete(using, keep_parents)
        dispatch.refresh_dictionary()
        return res

    class Meta:
        db_table = table_prefix + "system_dictionary"
        verbose_name = verbose_name_plural = "字典表"
        ordering = ("sort",)


class OperationLog(CoreModel):
    appid = models.CharField(
        max_length=255, verbose_name="应用id", null=True, blank=True, help_text="应用id", db_index=True
    )
    request_modular = models.CharField(
        max_length=64, verbose_name="请求模块", null=True, blank=True, help_text="请求模块", db_index=True
    )
    request_path = models.CharField(
        max_length=400, verbose_name="请求地址", null=True, blank=True, help_text="请求地址"
    )
    request_body = models.TextField(verbose_name="请求参数", null=True, blank=True, help_text="请求参数")
    request_method = models.CharField(
        max_length=8, verbose_name="请求方式", null=True, blank=True, help_text="请求方式", db_index=True
    )
    request_msg = models.TextField(verbose_name="操作说明", null=True, blank=True, help_text="操作说明")
    request_ip = models.CharField(
        max_length=32, verbose_name="请求ip地址", null=True, blank=True, help_text="请求ip地址", db_index=True
    )
    request_browser = models.CharField(
        max_length=64, verbose_name="请求浏览器", null=True, blank=True, help_text="请求浏览器"
    )
    response_code = models.CharField(
        max_length=32, verbose_name="响应状态码", null=True, blank=True, help_text="响应状态码", db_index=True
    )
    request_os = models.CharField(max_length=64, verbose_name="操作系统", null=True, blank=True, help_text="操作系统")
    json_result = models.TextField(verbose_name="返回信息", null=True, blank=True, help_text="返回信息")
    status = models.BooleanField(default=False, verbose_name="响应状态", help_text="响应状态", db_index=True)

    class Meta:
        db_table = table_prefix + "system_operation_log"
        verbose_name = verbose_name_plural = "操作日志"
        ordering = ("-create_datetime",)


class FolderList(CoreModel):
    name = models.CharField(
        max_length=200, null=True, blank=True, verbose_name="文件夹名称", help_text="文件夹名称", db_index=True
    )
    size = models.BigIntegerField(default=0, blank=True, verbose_name="文件夹大小", help_text="文件夹大小")

    class Meta:
        db_table = table_prefix + "system_folder_list"
        verbose_name = verbose_name_plural = "文件夹管理"
        ordering = ("-create_datetime",)
        unique_together = ("name",)


class FileList(CoreModel):
    folder = models.ForeignKey(
        to="FolderList",
        verbose_name="所属文件夹",
        on_delete=models.CASCADE,
        db_constraint=False,
        null=True,
        blank=True,
        help_text="所属文件夹",
    )
    name = models.CharField(max_length=200, null=True, blank=True, verbose_name="名称", help_text="名称", db_index=True)
    url = models.FileField(
        upload_to=media_file_name, null=True, blank=True, verbose_name="文件http地址", help_text="文件http地址"
    )
    file_url = models.CharField(max_length=255, blank=True, verbose_name="文件地址", help_text="文件地址")
    engine = models.CharField(max_length=100, default="local", blank=True, verbose_name="引擎", help_text="引擎")
    mime_type = models.CharField(max_length=100, blank=True, verbose_name="Mime类型", help_text="Mime类型")
    size = models.BigIntegerField(default=0, blank=True, verbose_name="文件大小", help_text="文件大小")
    md5sum = models.CharField(max_length=36, blank=True, verbose_name="文件md5", help_text="文件md5", db_index=True)

    @classmethod
    def save_file(cls, request, file_path, file_name, mime_type):
        # 保存到File model中
        instance = FileList()
        instance.name = file_name
        instance.engine = dispatch.get_system_config_values("file_storage.file_engine") or "local"
        instance.file_url = os.path.join(file_path, file_name)
        instance.mime_type = mime_type
        # 检查用户是否是匿名用户，避免创建匿名用户记录
        if request.user and not isinstance(request.user, AnonymousUser):
            instance.creator = request.user.id
            instance.modifier = request.user.id
            instance.dept_belong_id = request.user.dept_id

        file_backup = dispatch.get_system_config_values("file_storage.file_backup")
        dispatch.get_system_config_values("file_storage.file_engine") or "local"
        if file_backup:
            instance.url = os.path.join(file_path.replace(f"{settings.MEDIA_ROOT}/", ""), file_name)
        instance.url = os.path.join(file_path.replace(f"{settings.MEDIA_ROOT}/", ""), file_name)
        instance.save()
        return instance

    def save(self, *args, **kwargs):
        if not self.md5sum and self.url:  # file is new
            md5 = hashlib.md5()
            for chunk in self.url.chunks():
                md5.update(chunk)
            self.md5sum = md5.hexdigest()
        if not self.size and self.url:
            self.size = self.url.size
        if not self.file_url:
            url = media_file_name(self, self.name)
            self.file_url = f"/{settings.MEDIA_ROOT}/{url}"
        super().save(*args, **kwargs)
        # 解决相同文件名称_*的问题
        if self.url:
            rel_path = (self.url.name or "").strip().replace("\\", "/")
            if rel_path:
                media_url = (getattr(settings, "MEDIA_URL", "") or "/media/").strip().rstrip("/")
                self.file_url = f"/{media_url}/{rel_path}".replace("//", "/")
                FileList.objects.filter(pk=self.pk).update(file_url=self.file_url)
    class Meta:
        db_table = table_prefix + "system_file_list"
        verbose_name = verbose_name_plural = "文件管理"
        ordering = ("-create_datetime",)


class ChunkUploadSession(CoreModel):
    """
    断点续传：切片上传会话，记录 upload_id 与已上传的切片序号，供前端查询续传。
    """

    upload_id = models.CharField(
        max_length=64, unique=True, db_index=True, verbose_name="上传会话ID", help_text="上传会话ID"
    )
    file_md5 = models.CharField(max_length=32, db_index=True, verbose_name="文件MD5", help_text="文件MD5")
    filename = models.CharField(max_length=255, blank=True, verbose_name="文件名", help_text="文件名")
    total_size = models.BigIntegerField(default=0, verbose_name="文件总大小", help_text="文件总大小")
    chunk_total = models.IntegerField(default=0, verbose_name="切片总数", help_text="切片总数")
    uploaded_chunk_indices = models.JSONField(
        default=list,
        verbose_name="已上传切片序号",
        help_text="已上传的 chunk_index 列表，如 [0,1,2]",
    )

    class Meta:
        db_table = table_prefix + "system_chunk_upload_session"
        verbose_name = verbose_name_plural = "切片上传会话"
        ordering = ("-create_datetime",)


class Area(CoreModel):
    name = models.CharField(max_length=100, verbose_name="名称", help_text="名称")
    code = models.CharField(max_length=20, verbose_name="地区编码", help_text="地区编码", unique=True, db_index=True)
    level = models.BigIntegerField(
        verbose_name="地区层级(1省份 2城市 3区县 4乡级)", help_text="地区层级(1省份 2城市 3区县 4乡级)"
    )
    pinyin = models.CharField(max_length=255, verbose_name="拼音", help_text="拼音")
    initials = models.CharField(max_length=20, verbose_name="首字母", help_text="首字母")
    enable = models.BooleanField(default=True, verbose_name="是否启用", help_text="是否启用")
    pcode = models.ForeignKey(
        to="self",
        verbose_name="父地区编码",
        to_field="code",
        on_delete=models.PROTECT,
        db_constraint=False,
        null=True,
        blank=True,
        help_text="父地区编码",
    )

    class Meta:
        db_table = table_prefix + "system_area"
        verbose_name = verbose_name_plural = "地区表"
        ordering = ("code",)

    def __str__(self):
        return f"{self.name}"


class ApiWhiteList(CoreModel):
    url = models.CharField(max_length=200, help_text="url地址", verbose_name="url", db_index=True)
    METHOD_CHOICES = (
        (0, "GET"),
        (1, "POST"),
        (2, "PUT"),
        (3, "DELETE"),
    )
    method = models.IntegerField(
        default=0, verbose_name="接口请求方法", null=True, blank=True, help_text="接口请求方法"
    )
    enable_datasource = models.BooleanField(
        default=True, verbose_name="激活数据权限", help_text="激活数据权限", blank=True
    )

    class Meta:
        db_table = table_prefix + "api_white_list"
        verbose_name = verbose_name_plural = "接口白名单"
        ordering = ("-create_datetime",)


class SystemConfig(CoreModel):
    parent = models.ForeignKey(
        to="self",
        verbose_name="父级",
        on_delete=models.PROTECT,
        db_constraint=False,
        null=True,
        blank=True,
        help_text="父级",
    )
    title = models.CharField(max_length=50, verbose_name="标题", help_text="标题", db_index=True)
    key = models.CharField(max_length=200, verbose_name="键", help_text="键", db_index=True)
    value = models.JSONField(max_length=500, verbose_name="值", help_text="值", null=True, blank=True)
    sort = models.IntegerField(default=0, verbose_name="排序", help_text="排序", blank=True)
    status = models.BooleanField(default=True, verbose_name="启用状态", help_text="启用状态", db_index=True)
    data_options = models.JSONField(verbose_name="数据options", help_text="数据options", null=True, blank=True)
    FORM_ITEM_TYPE_LIST = (
        (0, "text"),
        (1, "datetime"),
        (2, "date"),
        (3, "textarea"),
        (4, "select"),
        (5, "checkbox"),
        (6, "radio"),
        (7, "img"),
        (8, "file"),
        (9, "switch"),
        (10, "number"),
        (11, "array"),
        (12, "imgs"),
        (13, "foreignkey"),
        (14, "manytomany"),
        (15, "time"),
        (16, "code"),
        (17, "role"),
        (18, "request"),
        (19, "response"),
    )
    form_item_type = models.IntegerField(
        choices=FORM_ITEM_TYPE_LIST, verbose_name="字段类型", help_text="字段类型", default=0, blank=True
    )
    rule = models.JSONField(null=True, blank=True, verbose_name="校验规则", help_text="校验规则")
    placeholder = models.CharField(max_length=100, null=True, blank=True, verbose_name="提示信息", help_text="提示信息")
    setting = models.JSONField(null=True, blank=True, verbose_name="配置", help_text="配置")

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        super().save(force_insert, force_update, using, update_fields)
        dispatch.refresh_system_config()  # 有更新则刷新系统配置

    def delete(self, using=None, keep_parents=False):
        res = super().delete(using, keep_parents)
        dispatch.refresh_system_config()
        return res

    class Meta:
        db_table = table_prefix + "system_config"
        verbose_name = verbose_name_plural = "系统配置表"
        ordering = ("sort",)
        unique_together = (("key", "parent_id"),)

    def __str__(self):
        return f"{self.title}"


class LoginLog(CoreModel):
    LOGIN_TYPE_CHOICES = ((1, "默认登录"), (2, "SSO登录"), (3, "移动端"), (4, "授群码登录"), (900, "自定义登录"))
    username = models.CharField(
        max_length=150, verbose_name="登录用户名", null=True, blank=True, help_text="登录用户名", db_index=True
    )
    ip = models.CharField(
        max_length=32, verbose_name="登录ip", null=True, blank=True, help_text="登录ip", db_index=True
    )
    agent = models.TextField(verbose_name="agent信息", null=True, blank=True, help_text="agent信息")
    browser = models.CharField(max_length=200, verbose_name="浏览器名", null=True, blank=True, help_text="浏览器名")
    os = models.CharField(max_length=200, verbose_name="操作系统", null=True, blank=True, help_text="操作系统")
    continent = models.CharField(max_length=50, verbose_name="州", null=True, blank=True, help_text="州")
    country = models.CharField(max_length=50, verbose_name="国家", null=True, blank=True, help_text="国家")
    province = models.CharField(max_length=50, verbose_name="省份", null=True, blank=True, help_text="省份")
    city = models.CharField(max_length=50, verbose_name="城市", null=True, blank=True, help_text="城市")
    district = models.CharField(max_length=50, verbose_name="县区", null=True, blank=True, help_text="县区")
    isp = models.CharField(max_length=50, verbose_name="运营商", null=True, blank=True, help_text="运营商")
    area_code = models.CharField(max_length=50, verbose_name="区域代码", null=True, blank=True, help_text="区域代码")
    country_english = models.CharField(
        max_length=50, verbose_name="英文全称", null=True, blank=True, help_text="英文全称"
    )
    country_code = models.CharField(max_length=50, verbose_name="简称", null=True, blank=True, help_text="简称")
    longitude = models.CharField(max_length=50, verbose_name="经度", null=True, blank=True, help_text="经度")
    latitude = models.CharField(max_length=50, verbose_name="纬度", null=True, blank=True, help_text="纬度")
    login_type = models.IntegerField(
        default=1, choices=LOGIN_TYPE_CHOICES, verbose_name="登录类型", help_text="登录类型", db_index=True
    )

    class Meta:
        db_table = table_prefix + "system_login_log"
        verbose_name = verbose_name_plural = "登录日志"
        ordering = ("-create_datetime",)


class Apps(CoreModel):
    agent_id = models.CharField(
        max_length=255, verbose_name="应用编号", null=False, blank=False, help_text="应用编号", default=create_agent_id
    )
    secret = EncrypyField(
        max_length=255, verbose_name="应用密钥", null=False, blank=False, help_text="应用密钥", default=create_secret
    )
    title = models.CharField(
        verbose_name="应用名称",
        max_length=255,
        default="",
        null=False,
        blank=False,
        help_text="应用名称",
        db_index=True,
    )
    icon = models.CharField(
        verbose_name="应用图标", max_length=255, default="", null=False, blank=False, help_text="应用图标"
    )
    old_title = models.CharField(
        verbose_name="曾用名", max_length=255, default="", null=True, blank=True, help_text="曾用名"
    )
    sort = models.IntegerField(default=1, verbose_name="显示排序", null=True, blank=True, help_text="显示排序")
    apps_type = models.IntegerField(
        choices=dispatch.get_dictionary_choices("apps_type"),
        verbose_name="应用类型",
        default=0,
        null=True,
        blank=True,
        help_text="应用类型",
    )
    open_type = models.IntegerField(
        choices=((0, "新打开浏览器页面"), (1, "弹框显示")),
        verbose_name="应用类型",
        default=0,
        null=True,
        blank=True,
        help_text="应用类型",
    )
    visible_range = CoreBaseAuthField(
        verbose_name="可见范围", null=True, blank=True, default=dict, help_text="可见范围"
    )
    server_ip = models.TextField(
        verbose_name="服务器IP白名单", help_text="服务器IP白名单", null=True, blank=True, default=""
    )
    healthy = models.CharField(
        verbose_name="健康检查", max_length=255, default="/healthy/", null=True, blank=True, help_text="健康检查"
    )
    readiness = models.CharField(
        verbose_name="存活中间件", max_length=255, default="/readiness/", null=True, blank=True, help_text="存活中间件"
    )
    apps_zip = models.CharField(
        verbose_name="项目文档", max_length=255, default="", null=True, blank=True, help_text="项目文档"
    )
    create_or_update_callback_url = models.TextField(
        verbose_name="新增修改", help_text="新增/修改/回调地址", null=True, blank=True, default=""
    )
    delete_callback_url = models.TextField(
        verbose_name="删除用户", help_text="删除回调地址", null=True, blank=True, default=""
    )
    desc = models.TextField(verbose_name="描述", default="", null=True, blank=True, help_text="描述")

    # 自动创建第三方api用户 不行 权限如何控制

    class Meta:
        db_table = table_prefix + "apps"
        verbose_name = verbose_name_plural = "应用管理"
        ordering = ("-id",)
        unique_together = ("title",)

    def __str__(self):
        return self.title


class SendMessageTask(CoreModel):
    apps = models.ForeignKey(
        Apps, verbose_name="开放应用", help_text="开放应用", null=True, blank=True, on_delete=models.CASCADE
    )
    send_type = models.IntegerField(
        verbose_name="发送类型",
        null=True,
        blank=True,
        help_text="发送类型",
        default=0,
        choices=((0, "邮件"), (1, "客户端通知")),
    )
    task_name = models.CharField(
        verbose_name="任务名称", max_length=255, help_text="任务名称", blank=True, null=True, default=""
    )
    task_id = models.CharField(
        verbose_name="任务id", max_length=255, help_text="任务id", blank=True, null=True, default=""
    )

    class Meta:
        db_table = table_prefix + "send_message_task"
        verbose_name = verbose_name_plural = "发送任务"
        ordering = ("-id",)

    def __str__(self):
        return self.task_id


class MessageCenter(CoreModel):
    send_message_task = models.ForeignKey(
        SendMessageTask, verbose_name="开放应用", help_text="开放应用", null=True, blank=True, on_delete=models.CASCADE
    )
    title = models.CharField(max_length=100, verbose_name="标题", help_text="标题")
    short_content = models.TextField(verbose_name="简要描述", help_text="简要描述", blank=False, null=False, default="")
    thumbnail = models.TextField(verbose_name="缩略图", help_text="缩略图", blank=True, null=True, default="")
    message_type = models.IntegerField(
        default=0, choices=dispatch.get_dictionary_choices("message_type"), db_index=True
    )
    file_list = models.JSONField(verbose_name="邮件附件", default=dict, null=True, blank=True, help_text="邮件附件")
    content = models.TextField(verbose_name="内容", help_text="内容", blank=True, null=True, default="")
    url = models.TextField(verbose_name="跳转地址", help_text="跳转地址", blank=True, null=True, default="")
    target_type = models.IntegerField(
        default=0,
        choices=dispatch.get_dictionary_choices("target_type"),
        verbose_name="目标类型",
        help_text="目标类型",
        db_index=True,
    )
    is_withdraw = models.BooleanField(
        default=False, blank=True, null=True, verbose_name="是否撤回", help_text="是否撤回", db_index=True
    )
    task_id = models.CharField(
        verbose_name="任务id", max_length=255, help_text="任务id", blank=True, null=True, default=""
    )

    send_user_info = CoreBaseAuthField(
        verbose_name="发送范围", null=True, blank=True, default=dict, help_text="发送范围"
    )
    # 定时发送
    periodic_task = models.ManyToManyField(
        PeriodicTask,
        verbose_name="定时发送",
        help_text="定时发送",
        blank=True,
        related_name="many_periodic_send_task",
        db_constraint=False,
    )
    expires = models.DateTimeField(blank=True, null=True, verbose_name="过期时间", help_text="过期时间")
    ext_kwargs = models.JSONField(
        verbose_name="发送消息扩展参数", null=False, blank=False, default=dict, help_text="发送消息扩展参数"
    )

    class Meta:
        db_table = table_prefix + "message_center"
        verbose_name = verbose_name_plural = "消息中心"
        ordering = ("-create_datetime",)


class MessageCenterTargetUser(CoreModel):
    users = models.ForeignKey(
        Users,
        related_name="target_user",
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="关联用户表",
        help_text="关联用户表",
    )
    messagecenter = models.ForeignKey(
        MessageCenter,
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="关联消息中心表",
        help_text="关联消息中心表",
    )
    is_send = models.BooleanField(
        default=False, blank=True, null=True, verbose_name="是否推送", help_text="是否推送", db_index=True
    )
    is_read = models.BooleanField(
        default=False, blank=True, null=True, verbose_name="是否已读", help_text="是否已读", db_index=True
    )
    send_mails_result = models.CharField(
        verbose_name="发送结果", max_length=255, help_text="发送结果仅用于异步通知", blank=True, null=True, default=""
    )

    class Meta:
        db_table = table_prefix + "message_center_target_user"
        verbose_name = verbose_name_plural = "消息中心目标用户表"


# 任务管理基础版
class BaseTask(CoreModel):
    code = models.CharField(
        verbose_name="任务编号",
        max_length=255,
        default=newuuid,
        null=True,
        blank=True,
        help_text="任务编号",
        db_index=True,
    )
    task_name = models.CharField(
        verbose_name="任务名称",
        max_length=255,
        default="",
        null=False,
        blank=False,
        help_text="任务名称",
        db_index=True,
    )
    task_type = models.IntegerField(
        choices=dispatch.get_dictionary_choices("task_type"),
        verbose_name="任务类型",
        default=0,
        null=False,
        blank=False,
        help_text="任务类型",
        db_index=True,
    )
    periodic_task = models.ManyToManyField(
        PeriodicTask,
        verbose_name="周期性任务",
        help_text="周期性任务",
        blank=True,
        related_name="base_task_periodic_task",
    )
    task_status = models.BooleanField(default=False, verbose_name="任务状态", help_text="任务状态", db_index=True)
    request_config = models.JSONField(
        verbose_name="请求配置", max_length=50000, default=dict, null=True, blank=True, help_text="请求配置"
    )
    last_run_time = models.DateTimeField(
        blank=True, null=True, verbose_name="最后一次运行时间", help_text="最后一次运行时间"
    )
    task_desc = models.TextField(verbose_name="任务描述", default="", null=True, blank=True, help_text="任务描述")

    class Meta:
        db_table = table_prefix + "base_task"
        verbose_name = verbose_name_plural = "任务管理"

        ordering = ("-id",)
        # unique_together = ('task_name',)

    def __str__(self):
        return self.task_name


class TaskRunHistory(CoreModel):
    base_task = models.ForeignKey(
        BaseTask,
        on_delete=models.CASCADE,
        db_constraint=False,
        null=True,
        blank=True,
        verbose_name="触发任务",
        help_text="触发任务",
    )
    url_name = models.CharField(
        verbose_name="url命名", max_length=255, default="", null=False, blank=False, help_text="url命名"
    )
    start_time = models.DateTimeField(
        blank=True, null=True, verbose_name="任务请求开始时间", help_text="请求开始时间", db_index=True
    )
    end_time = models.DateTimeField(blank=True, null=True, verbose_name="任务请求结束时间", help_text="请求结束时间")
    status_code = models.IntegerField(
        verbose_name="请求状态响应码", default=0, help_text="请求状态响应码", db_index=True
    )
    request_config = models.JSONField(
        verbose_name="请求配置", max_length=50000, default=dict, null=True, blank=True, help_text="请求配置"
    )
    run_result = models.TextField(
        verbose_name="运行返回结果", default=dict, null=True, blank=True, help_text="运行返回结果"
    )
    periodic_task = models.ForeignKey(
        to=PeriodicTask,
        on_delete=models.SET_NULL,
        db_constraint=False,
        related_name="periodic",
        verbose_name="执行任务定时器",
        help_text="执行任务定时器",
        null=True,
        blank=True,
    )

    class Meta:
        db_table = table_prefix + "task_run_history"
        verbose_name = verbose_name_plural = "任务运行历史"
        ordering = ("-id",)


# 第三方服务依赖
class ExternalRely(CoreModel):
    apps = models.ForeignKey(
        Apps,
        verbose_name="开放应用",
        help_text="开放应用",
        null=True,
        blank=True,
        db_constraint=False,
        on_delete=models.CASCADE,
    )
    title = models.CharField(
        verbose_name="应用名称", max_length=255, default="", null=True, blank=True, help_text="应用名称"
    )
    ip = models.CharField(
        verbose_name="服务域名", max_length=255, default="", null=False, blank=False, help_text="服务域名"
    )
    port = models.IntegerField(verbose_name="端口", default=80, null=False, blank=False, help_text="端口")
    url = models.TextField(
        verbose_name="url地址", max_length=50000, default="", null=False, blank=False, help_text="url地址"
    )

    class Meta:
        db_table = table_prefix + "external_rely"
        verbose_name = verbose_name_plural = "第三方服务依赖"
        ordering = ("-id",)

    def __str__(self):
        return self.title


# 埋点
# WHO（什么人），When（在什么时间），Where（在什么地点），How（以什么方式），What（干了什么事情）
class AppsBuriedPoint(CoreModel):
    apps = models.ForeignKey(
        Apps,
        verbose_name="开放应用",
        help_text="开放应用",
        null=True,
        blank=True,
        db_constraint=False,
        on_delete=models.CASCADE,
    )


class AIDeskAppManage(CoreModel):
    agent_id = models.CharField(
        max_length=255, verbose_name="应用编号", null=False, blank=False, help_text="应用编号", default=create_agent_id
    )
    secret = EncrypyField(
        max_length=255, verbose_name="应用密钥", null=False, blank=False, help_text="应用密钥", default=create_secret
    )
    dept = models.ForeignKey(
        to="Dept",
        verbose_name="所属部门",
        on_delete=models.PROTECT,
        db_constraint=False,
        null=True,
        blank=True,
        help_text="所属部门",
    )
    avatar = models.CharField(max_length=255, verbose_name="头像", null=True, blank=True, help_text="头像")
    name = models.CharField(
        verbose_name="名称", max_length=255, default="", null=False, blank=False, help_text="名称", db_index=True
    )
    alias = models.CharField(max_length=255, verbose_name="别名", help_text="别名", null=True, blank=True)
    email = models.EmailField(
        max_length=255, verbose_name="邮箱", null=True, blank=True, help_text="邮箱", db_index=True
    )
    mobile = EncrypyField(max_length=255, verbose_name="手机", null=True, blank=True, help_text="手机")
    landline = models.CharField(max_length=255, verbose_name="座机", null=True, blank=True, help_text="固定电话")
    welcome_message = models.TextField(verbose_name="欢迎语", default="", null=True, blank=True, help_text="欢迎语")
    sort = models.IntegerField(default=1, verbose_name="显示排序", null=True, blank=True, help_text="显示排序")
    allocation_type = models.IntegerField(
        verbose_name="分配方式",
        null=True,
        blank=True,
        help_text="分配方式",
        default=0,
        choices=dispatch.get_dictionary_choices("im_chat_app_manage_stype_choices"),
    )
    is_release = models.BooleanField(
        verbose_name="是否发布", default=False, null=True, blank=True, help_text="是否发布", db_index=True
    )
    visibility = CoreBaseAuthField(verbose_name="可见范围", null=True, blank=True, default=dict, help_text="可见范围")
    manager = CoreBaseAuthField(verbose_name="管理者", null=True, blank=True, default=dict, help_text="管理者")
    seats = CoreBaseAuthField(verbose_name="坐席", null=True, blank=True, default=dict, help_text="坐席")
    ai_switch = models.BooleanField(
        verbose_name="开启AI", default=False, null=True, blank=True, help_text="开启AI 默认False 不开启"
    )
    recommended_questions = models.BooleanField(
        verbose_name="推荐问题", default=False, null=True, blank=True, help_text="动态推荐用户可能会问的相似问题"
    )
    dissatisfied_switch = models.BooleanField(
        verbose_name="询问不满意原因", default=True, null=True, blank=True, help_text="开启满意度调查 默认True 开启"
    )
    dissatisfied = models.JSONField(
        verbose_name="不满意Json", default=dict, null=True, blank=True, help_text="不满意Json"
    )
    ai_settings_json = models.JSONField(
        verbose_name="API请求Json", default=dict, null=True, blank=True, help_text="API请求Json"
    )
    human_services = models.BooleanField(
        verbose_name="人工服务",
        default=False,
        null=True,
        blank=True,
        help_text="客户咨询时将先根据录入的知识库自动回复客户，无法回复时再转到人工服务",
    )
    desc = models.TextField(verbose_name="描述", default="", null=True, blank=True, help_text="描述")

    class Meta:
        db_table = table_prefix + "ai_desk_app_manage"
        verbose_name = verbose_name_plural = "员工服务"
        ordering = ("-id",)
        unique_together = ("name",)

    def __str__(self):
        return self.name


class IMChatQA(CoreModel):
    ai_desk_app_manage = models.ForeignKey(
        AIDeskAppManage,
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="员工服务",
        help_text="员工服务",
    )
    question = models.TextField(verbose_name="答案内容", help_text="答案内容", blank=False, null=False, default="")
    answer = models.TextField(verbose_name="答案内容", help_text="答案内容", blank=False, null=False, default="")
    q_type = models.IntegerField(
        verbose_name="类型",
        choices=((0, "热门问题"), (1, "AI"), (2, "未知")),
        null=True,
        blank=True,
        help_text="类型",
        default=0,
        db_index=True,
    )
    is_it_solved = models.IntegerField(
        verbose_name="是否解决",
        choices=((0, "解决"), (1, "未解决"), (2, "未选择")),
        help_text="是否解决",
        blank=True,
        null=True,
        default=2,
        db_index=True,
    )

    class Meta:
        db_table = table_prefix + "im_chat_qa"
        verbose_name = verbose_name_plural = "QA问答记录"


class IMChatQuestionGroup(CoreModel):
    ai_desk_app_manage = models.ForeignKey(
        AIDeskAppManage,
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="员工服务",
        help_text="员工服务",
    )
    parent = models.ForeignKey(
        to="IMChatQuestionGroup",
        on_delete=models.CASCADE,
        verbose_name="上级",
        null=True,
        blank=True,
        db_constraint=False,
        help_text="上级",
    )
    title = models.CharField(
        max_length=255,
        verbose_name="分组名称",
        help_text="分组名称",
        null=False,
        blank=False,
        default="",
        db_index=True,
    )
    sort = models.IntegerField(default=1, verbose_name="排序", help_text="排序")

    class Meta:
        db_table = table_prefix + "im_chat_question_group"
        verbose_name = verbose_name_plural = "员工服务热门问题分组"


class IMChatQuestion(CoreModel):
    im_chat_question_group = models.ForeignKey(
        IMChatQuestionGroup,
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="热门问题分组",
        help_text="热门问题分组",
    )
    parent = models.ForeignKey(
        to="IMChatQuestion",
        on_delete=models.CASCADE,
        verbose_name="上级",
        null=True,
        blank=True,
        db_constraint=False,
        help_text="上级",
    )
    title = models.CharField(
        max_length=255,
        verbose_name="问题标题",
        help_text="问题标题",
        null=False,
        blank=False,
        default="",
        db_index=True,
    )
    content = models.TextField(verbose_name="答案内容", help_text="答案内容", blank=False, null=False, default="")
    is_enable = models.BooleanField(
        verbose_name="是否启用", help_text="是否启用", blank=True, null=True, default=True, db_index=True
    )
    hit = models.IntegerField(verbose_name="点击量", help_text="点击量", blank=True, null=True, default=0)

    class Meta:
        db_table = table_prefix + "im_chat_question"
        verbose_name = verbose_name_plural = "员工服务热门问题"
        ordering = (
            "-hit",
            "id",
        )


class Collaborator(CoreModel):
    user_info = CoreBaseAuthField(verbose_name="协作人", null=True, blank=True, default=dict, help_text="协作人")

    class Meta:
        db_table = table_prefix + "collaborator"
        verbose_name = verbose_name_plural = "协作人"

        ordering = ("-create_datetime",)


class IMSession(CoreModel):
    parent = models.ForeignKey(
        to="IMSession",
        on_delete=models.SET_NULL,
        default=None,
        verbose_name="父级会话ID",
        db_constraint=False,
        null=True,
        blank=True,
        help_text="父级会话ID",
        db_index=True,
    )
    session_id = models.CharField(
        max_length=255, unique=True, null=True, blank=True, verbose_name="会话id", help_text="会话id", default=newuuid
    )
    from_user = models.ForeignKey(
        to="Users",
        db_constraint=False,
        on_delete=models.CASCADE,
        verbose_name="会话用户",
        help_text="会话用户",
        null=True,
    )
    session_type = models.BooleanField(
        default=False, verbose_name="会话类型", help_text="会话类型，0 点对点 1群聊", db_index=True
    )
    is_it_pinned = models.BooleanField(
        default=False, verbose_name="是否置顶", help_text="是否置顶，True置顶 False不置顶", db_index=True
    )
    is_claim = models.IntegerField(
        verbose_name="是否认领",
        choices=((0, "未认领"), (1, "已认领"), (2, "消息免打扰")),
        help_text="是否认领",
        null=True,
        blank=True,
        default=0,
        db_index=True,
    )
    is_colse = models.BooleanField(
        verbose_name="是否关闭", help_text="是否关闭", null=True, blank=True, default=False, db_index=True
    )
    ai_desk_app_manage = models.ForeignKey(
        AIDeskAppManage,
        null=True,
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="员工服务",
        help_text="员工服务",
    )

    class Meta:
        db_table = table_prefix + "im_session"
        verbose_name = verbose_name_plural = "会话表"
        ordering = ("-id", "-is_it_pinned")

    def __str__(self):
        return self.session_id


class IMChatRecordForm(CoreModel):
    im_session = models.OneToOneField(
        IMSession,
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="会话",
        help_text="会话",
    )
    first_reply_time = models.DateTimeField(
        verbose_name="首次回复时间", null=True, blank=True, help_text="首次回复时间"
    )
    reply_interval = models.IntegerField(
        verbose_name="回复间隔", null=True, blank=True, help_text="回复间隔", default=0
    )
    satisfaction_type = models.IntegerField(
        choices=dispatch.get_dictionary_choices("satisfaction_type"),
        verbose_name="满意度",
        null=True,
        blank=True,
        help_text="满意度",
        default=5,
    )
    satisfaction_info = models.TextField(
        verbose_name="反馈内容", help_text="反馈内容", blank=False, null=False, default=""
    )

    class Meta:
        db_table = table_prefix + "im_chat_record_form"
        verbose_name = verbose_name_plural = "满意度"
        ordering = ("id",)


# 点对点
class IMChatToUser(CoreModel):
    im_session = models.ForeignKey(
        to="IMSession",
        db_constraint=False,
        related_name="SessionFromUser",
        on_delete=models.CASCADE,
        verbose_name="会话",
        help_text="会话",
    )
    from_user = models.ForeignKey(
        to="Users",
        db_constraint=False,
        related_name="FromUser",
        on_delete=models.CASCADE,
        verbose_name="发送用户",
        help_text="发送用户",
    )
    from_to_user = models.ForeignKey(
        to="Users",
        db_constraint=False,
        related_name="FromToUser",
        on_delete=models.PROTECT,
        verbose_name="接收用户",
        help_text="接收用户",
    )
    msg_stype = models.IntegerField(
        verbose_name="消息类型",
        help_text="消息类型",
        null=True,
        choices=dispatch.get_dictionary_choices("im_stype_choices"),
        blank=True,
        default=0,
        db_index=True,
    )
    content = models.JSONField(verbose_name="内容", help_text="内容", blank=True, null=True, default=dict)

    class Meta:
        db_table = table_prefix + "im_chat_to_user"
        verbose_name = verbose_name_plural = "点对点聊天"
        ordering = ("id",)


# 群聊
class IMChatToGroup(CoreModel):
    im_session = models.ForeignKey(
        IMSession,
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="会话",
        help_text="会话",
    )
    name = models.CharField(
        max_length=255, null=True, blank=True, verbose_name="群聊名称", help_text="群聊名称", db_index=True
    )
    # 群聊管理员
    group_admins = models.ManyToManyField(
        Users, db_constraint=False, related_name="ImGroupAdmins", verbose_name="群管理员", help_text="群管理员"
    )
    group_chat_avatar = models.CharField(
        max_length=255, default="", null=True, blank=True, verbose_name="群聊头像", help_text="群聊头像"
    )
    group_ext_info = models.JSONField(
        verbose_name="扩展字段", help_text="扩展字段", blank=True, null=True, default=dict
    )
    # 群成员
    from_many_to_user = models.ManyToManyField(
        Users,
        db_constraint=False,
        related_name="ManyGroupMembers",
        verbose_name="接收用户",
        help_text="接收用户",
    )

    class Meta:
        db_table = table_prefix + "im_chat_to_group"
        verbose_name = verbose_name_plural = "群聊天"
        ordering = ("-id",)


class IMChatToGroupMessage(CoreModel):
    im_chat_to_group = models.ForeignKey(
        to="IMChatToGroup",
        on_delete=models.CASCADE,
        db_constraint=False,
        verbose_name="群聊天",
        help_text="群聊天",
    )
    from_user = models.ForeignKey(
        to="Users",
        db_constraint=False,
        related_name="GroupFromUser",
        on_delete=models.CASCADE,
        verbose_name="发送用户",
        help_text="发送用户",
    )
    from_to_user = models.ForeignKey(
        to="Users",
        db_constraint=False,
        related_name="GroupFromToUser",
        on_delete=models.PROTECT,
        verbose_name="接收用户",
        help_text="接收用户",
    )
    content = models.JSONField(verbose_name="内容", help_text="内容", blank=True, null=True, default=dict)
    msg_stype = models.IntegerField(
        verbose_name="消息类型",
        help_text="消息类型",
        null=True,
        choices=dispatch.get_dictionary_choices("im_stype_choices"),
        blank=True,
        default=0,
        db_index=True,
    )
    is_read = models.BooleanField(
        default=True, blank=True, null=True, verbose_name="是否已读", help_text="是否已读", db_index=True
    )

    class Meta:
        db_table = table_prefix + "im_chat_to_group_message"
        verbose_name = verbose_name_plural = "群聊天内容"
        ordering = ("-id",)


class SafetyCheckRecord(CoreModel):
    pkg_count = models.IntegerField(verbose_name="扫描包数", help_text="扫描包数", blank=True, null=True, default=0)
    vulnerabilities_count = models.IntegerField(
        verbose_name="漏洞包数量", help_text="漏洞包数量", blank=True, null=True, default=0
    )
    vulnerabilities = models.JSONField(
        max_length=50000, verbose_name="漏洞详情", help_text="漏洞详情", null=True, blank=True, default=dict
    )

    class Meta:
        db_table = table_prefix + "safety_check_record"
        verbose_name = verbose_name_plural = "安全漏洞检查记录"
        ordering = ("-id",)


class FrequentlyUsedContacts(CoreModel):
    user = models.ForeignKey(
        to="Users",
        on_delete=models.CASCADE,
        default=None,
        verbose_name="常用联系人",
        db_constraint=False,
        null=True,
        blank=True,
        help_text="常用联系人",
        db_index=True,
    )

    class Meta:
        db_table = table_prefix + "frequently_used_contacts"
        verbose_name = verbose_name_plural = "常用联系人"
        ordering = ("-id",)
        # unique_together = ('creator', 'user')


class SystemUsage(CoreModel):
    apps = models.ForeignKey(
        Apps,
        verbose_name="开放应用",
        help_text="开放应用",
        null=True,
        blank=True,
        db_constraint=False,
        on_delete=models.CASCADE,
    )
    use_count = models.IntegerField(default=0, verbose_name="使用总数", help_text="使用总数")
    usage_type = models.IntegerField(default=0, verbose_name="类型", help_text="类型", db_index=True)
    title = models.TextField(verbose_name="模块名称", help_text="模块名称", blank=False, null=False, default="")
    path = models.TextField(verbose_name="路径", help_text="路径", blank=False, null=False, default="")

    class Meta:
        db_table = table_prefix + "system_usage"
        verbose_name = verbose_name_plural = "统计量"
        ordering = ("-id",)




"""菜单按钮 MenuButtonViewSet 接口单元测试。"""

import pytest
from rest_framework import status


pytestmark = [pytest.mark.django_db]

API = "/base/api/system/menu_button"


def test_menu_button_list_ok(api_client):
    """GET /menu_button/ 可能无需认证，检查可访问。"""
    response = api_client.get(API + "/")
    assert response.status_code in (status.HTTP_200_OK, status.HTTP_401_UNAUTHORIZED)


def test_menu_button_list_authenticated_ok(authenticated_client):
    """GET /menu_button/ 已认证返回 200。"""
    response = authenticated_client.get(API + "/")
    assert response.status_code == status.HTTP_200_OK
    assert response.json().get("code") == 2000



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : operation_log.py
# @Author  : cx
# @Time    : 11/2/2026
# @Desc    : 操作日志查询与筛选。
"""
操作日志模块：提供操作日志的 CRUD、导出、使用量统计；
日志文件分析（analyze_logs）、下载（download_logs）、实时 tail 流（tail_logs）。
仅管理员可访问日志文件相关接口（download_logs / tail_logs）。
"""
import logging
import re
import os
import time
from datetime import datetime, timedelta
from urllib.parse import quote

from django.conf import settings
from django.db.models import Count
from django.http import StreamingHttpResponse
from rest_framework.decorators import action

from django_base_ai.system.models import Apps, OperationLog
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse
from django_base_ai.utils.permission import AdminPermission
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class OperationLogSerializer(CustomModelSerializer):
    """
    日志-序列化器
    """

    class Meta:
        model = OperationLog
        fields = "__all__"
        read_only_fields = ["id"]


class OperationLogCreateUpdateSerializer(CustomModelSerializer):
    """
    操作日志  创建/更新时的列化器
    """

    class Meta:
        model = OperationLog
        fields = "__all__"


class ExportOperationLogSerializer(CustomModelSerializer):
    """
    操作日志  创建/更新时的列化器
    """

    class Meta:
        model = OperationLog
        fields = "__all__"


class OperationLogViewSet(CustomModelViewSet):
    """
    操作日志接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = OperationLog.objects.order_by("-create_datetime")
    serializer_class = OperationLogSerializer
    search_fields = ["appid", "request_modular", "request_ip", "response_code"]
    # 导出
    export_field_label = {
        "appid": "应用id",
        "creator_name": "操作人",
        "request_modular": "请求模块",
        "request_path": "请求地址",
        "request_body": "请求参数",
        "request_method": "请求方式",
        "request_msg": "操作说明",
        "request_ip": "请求ip地址",
        "request_browser": "请求浏览器",
        "response_code": "响应状态码",
        "request_os": "操作系统",
        "json_result": "返回信息",
    }
    export_serializer_class = ExportOperationLogSerializer

    @action(methods=["get"], detail=False)
    def usage_amount(self, request, *args, **kwargs):
        """
        使用量统计：按 appid、request_method、response_code 聚合计数。
        :param request: DRF 请求
        :return: DetailResponse([])，统计结果可在此扩展返回
        """
        logger.debug("operation_log usage_amount 请求")
        operation_log_result = OperationLog.objects.values("appid").annotate(count=Count("id"))
        operation_action_result = OperationLog.objects.values("request_method").annotate(count=Count("id"))
        operation_code_result = OperationLog.objects.values("response_code").annotate(count=Count("id"))
        logger.debug("usage_amount appid=%s method=%s code=%s", list(operation_log_result), list(operation_action_result), list(operation_code_result))
        return DetailResponse([])

    def file_iterator(self, filename, chunk_size=512):
        """
        文件迭代器，按块读取文件并 yield，用于 StreamingHttpResponse 流式下载。
        :param filename: 文件绝对路径
        :param chunk_size: 每块字节数，默认 512
        """
        with open(filename, "rb") as f:
            while True:
                c = f.read(chunk_size)
                if c:
                    yield c
                else:
                    break

    @action(methods=["get"], detail=False)
    def analyze_logs(self, request, *args, **kwargs):
        """
        分析日志文件中的报错内容：按级别/模块/时间统计，返回错误模式与最近错误列表。
        查询参数：log_type(error|server)、start_date、end_date、error_level、limit。
        """
        import re
        from collections import Counter, defaultdict
        from datetime import datetime

        log_type = request.GET.get("log_type", "error")  # error 或 server
        logger.info("analyze_logs 请求 log_type=%s", log_type)
        start_date = request.GET.get("start_date")
        end_date = request.GET.get("end_date")
        error_level = request.GET.get("error_level", "ERROR")  # ERROR, WARNING, INFO
        limit = int(request.GET.get("limit", 100))  # 返回记录数限制

        # 构建日志文件路径
        logs_dir = os.path.join(settings.BASE_DIR, "logs")
        if log_type == "error":
            log_file_path = os.path.join(logs_dir, "error.log")
        else:
            log_file_path = os.path.join(logs_dir, "server.log")

        if not os.path.exists(log_file_path):
            logger.warning("analyze_logs 日志文件不存在 path=%s", log_file_path)
            return ErrorResponse(msg=f"日志文件不存在: {log_type}.log")

        error_logs = []
        error_patterns = []
        error_counts = Counter()
        module_counts = Counter()
        time_counts = defaultdict(int)

        try:
            with open(log_file_path, encoding="utf-8") as f:
                lines = f.readlines()

            current_error = None

            for line_num, line in enumerate(lines, 1):
                line = line.strip()
                if not line:
                    continue

                # 匹配日志格式: [时间][模块][级别] 消息
                log_match = re.match(r"\[([^\]]+)\]\[([^\]]+)\] \[([^\]]+)\] (.+)", line)

                if log_match:
                    timestamp_str, module, level, message = log_match.groups()

                    # 解析时间
                    try:
                        timestamp = datetime.strptime(timestamp_str, "%Y-%m-%d %H:%M:%S,%f")
                    except ValueError:
                        try:
                            timestamp = datetime.strptime(timestamp_str, "%Y-%m-%d %H:%M:%S")
                        except ValueError:
                            timestamp = None

                    # 过滤时间范围
                    if start_date and timestamp:
                        try:
                            start_dt = datetime.strptime(start_date, "%Y-%m-%d")
                            if timestamp.date() < start_dt.date():
                                continue
                        except ValueError:
                            pass

                    if end_date and timestamp:
                        try:
                            end_dt = datetime.strptime(end_date, "%Y-%m-%d")
                            if timestamp.date() > end_dt.date():
                                continue
                        except ValueError:
                            pass

                    # 处理错误级别
                    if level == error_level:
                        # 保存当前错误
                        if current_error:
                            error_logs.append(current_error)

                        current_error = {
                            "timestamp": timestamp_str,
                            "module": module,
                            "level": level,
                            "message": message,
                            "line_number": line_num,
                            "stack_trace": [],
                        }

                        # 统计
                        error_counts[message] += 1
                        module_counts[module] += 1
                        if timestamp:
                            time_counts[timestamp.strftime("%Y-%m-%d")] += 1

                    # 处理堆栈跟踪
                    elif current_error and (line.startswith("  File ") or line.startswith("Traceback")):
                        current_error["stack_trace"].append(line)

                # 处理多行错误消息
                elif current_error and not line.startswith("["):
                    current_error["message"] += " " + line

            # 添加最后一个错误
            if current_error:
                error_logs.append(current_error)

        except Exception as e:
            logger.exception("analyze_logs 读取日志文件失败 path=%s", log_file_path)
            return ErrorResponse(msg=f"读取日志文件失败: {str(e)}")

        error_logs = error_logs[-limit:] if limit > 0 else error_logs
        logger.info("analyze_logs 完成 path=%s total_errors=%s", log_file_path, len(error_logs))

        # 分析错误模式
        error_patterns = []
        for error_msg, count in error_counts.most_common(10):
            # 提取错误模式（去除具体参数）
            pattern = re.sub(r"\d+", "N", error_msg)  # 数字替换为N
            pattern = re.sub(r"[a-f0-9]{8,}", "HASH", pattern)  # 哈希值替换
            pattern = re.sub(r"/[^\s]+", "/PATH", pattern)  # 路径替换
            error_patterns.append({"pattern": pattern, "count": count, "example": error_msg})

        # 构建返回数据
        result_data = {
            "log_file": f"{log_type}.log",
            "summary": {
                "total_errors": len(error_logs),
                "unique_error_types": len(error_counts),
                "top_modules": dict(module_counts.most_common(5)),
                "date_range": {
                    "start": min(time_counts.keys()) if time_counts else None,
                    "end": max(time_counts.keys()) if time_counts else None,
                },
            },
            "error_patterns": error_patterns,
            "module_statistics": [
                {"module": module, "count": count} for module, count in module_counts.most_common(10)
            ],
            "time_statistics": dict(time_counts),
            "recent_errors": [
                {
                    "timestamp": log["timestamp"],
                    "module": log["module"],
                    "level": log["level"],
                    "message": log["message"],
                    "line_number": log["line_number"],
                    "stack_trace": log["stack_trace"][:5],  # 只返回前5行堆栈
                }
                for log in error_logs[-20:]  # 最近20条错误
            ],
        }

        return DetailResponse(data=result_data)

    @action(methods=["get"], detail=False, permission_classes=[AdminPermission])
    def download_logs(self, request, *args, **kwargs):
        """
        管理员下载日志文件（仅 AdminPermission）。
        仅允许 server.log / error.log，防止路径遍历。
        """
        log_filename = request.GET.get("filename", "server.log")
        logger.info("download_logs 请求 filename=%s user=%s", log_filename, getattr(request.user, "username", None))

        if not log_filename or log_filename not in ["server.log", "error.log"]:
            logger.warning("download_logs 无效文件名 filename=%s", log_filename)
            return ErrorResponse(msg="无效的日志文件名")

        # 构建日志文件路径
        logs_dir = os.path.join(settings.BASE_DIR, "logs")
        log_file_path = os.path.join(logs_dir, log_filename)

        if not os.path.exists(log_file_path):
            logger.warning("download_logs 文件不存在 path=%s", log_file_path)
            return ErrorResponse(msg="日志文件不存在")

        if not os.access(log_file_path, os.R_OK):
            logger.warning("download_logs 文件不可读 path=%s", log_file_path)
            return ErrorResponse(msg="无法读取日志文件")

        response = StreamingHttpResponse(self.file_iterator(log_file_path))
        response["Content-Type"] = "application/octet-stream"
        response["Content-Disposition"] = f'attachment; filename="{quote(log_filename)}"'

        return response

    # 日志行时间戳正则，与 analyze_logs 一致：[时间][模块][级别] 消息
    _LOG_LINE_TIME_RE = re.compile(r"^\[([^\]]+)\]\[([^\]]+)\] \[([^\]]+)\] (.+)$")

    def _parse_log_line_time(self, line):
        """解析日志行首的时间戳，返回 datetime 或 None。支持 %Y-%m-%d %H:%M:%S,%f 与 %Y-%m-%d %H:%M:%S。"""
        m = self._LOG_LINE_TIME_RE.match(line.strip())
        if not m:
            return None
        ts_str = m.group(1)
        for fmt in ("%Y-%m-%d %H:%M:%S,%f", "%Y-%m-%d %H:%M:%S"):
            try:
                return datetime.strptime(ts_str.strip(), fmt)
            except ValueError:
                continue
        return None

    def _tail_log_stream(
        self,
        log_file_path,
        from_beginning=False,
        poll_interval=0.5,
        encoding="utf-8",
        start_ts=None,
        end_ts=None,
        max_duration_seconds=None,
        max_lines=None,
    ):
        """
        类 tail -f：持续读取日志文件新增内容，按行以 SSE 格式 yield。
        可选按时间范围过滤（仅输出在 start_ts～end_ts 内的行，多行堆栈按首行时间判断）。
        可选 max_duration_seconds / max_lines 限制时长或行数后主动结束，避免 curl 等客户端一直挂起。
        :param log_file_path: 日志文件绝对路径
        :param from_beginning: True=从文件开头读起，False=从当前末尾开始（等同 tail -f）
        :param poll_interval: 无新内容时休眠秒数
        :param encoding: 文件编码
        :param start_ts: 仅输出时间 >= start_ts 的日志行（datetime，None 表示不限制）
        :param end_ts: 仅输出时间 <= end_ts 的日志行（datetime，None 表示不限制）
        :param max_duration_seconds: 最多流式传输秒数，超时后结束（None=不限制）
        :param max_lines: 最多输出日志行数（不含首行说明），达到后结束（None=不限制）
        """
        def _sse(line):
            payload = line.replace("\n", "\ndata: ")
            return f"data: {payload}\n\n".encode(encoding)

        time_range_desc = ""
        if start_ts or end_ts:
            time_range_desc = f", time_range=[{start_ts} ~ {end_ts}]"
        yield _sse(f"[tail] {os.path.basename(log_file_path)} (from_beginning={from_beginning}{time_range_desc})")
        yield _sse("")
        # runserver(wsgiref) 默认约 4KB 才刷新，首包加 SSE 注释填充使 curl 能立即看到输出
        yield (f": {' ' * 4092}\n\n".encode(encoding))

        deadline = None
        if max_duration_seconds is not None and max_duration_seconds > 0:
            deadline = time.monotonic() + max_duration_seconds
        line_count = 0

        def _in_time_range(ts):
            if ts is None:
                return True
            if start_ts is not None and ts < start_ts:
                return False
            if end_ts is not None and ts > end_ts:
                return False
            return True

        try:
            with open(log_file_path, "r", encoding=encoding, errors="replace") as f:
                if from_beginning:
                    f.seek(0)
                else:
                    f.seek(0, 2)
                last_size = f.tell()
                last_ts = None
                while True:
                    if deadline is not None and time.monotonic() >= deadline:
                        yield _sse("[tail] 已达 max_duration_seconds，结束流")
                        return
                    if max_lines is not None and line_count >= max_lines:
                        yield _sse(f"[tail] 已达 max_lines={max_lines}，结束流")
                        return

                    line = f.readline()
                    if line:
                        line = line.rstrip("\n\r")
                        if not line:
                            continue
                        ts = self._parse_log_line_time(line)
                        if ts is not None:
                            last_ts = ts
                        if _in_time_range(last_ts if ts is None else ts):
                            yield _sse(line)
                            line_count += 1
                    else:
                        try:
                            curr = os.path.getsize(log_file_path)
                            if curr < last_size:
                                f.seek(0)
                            last_size = curr
                        except OSError:
                            pass
                        time.sleep(poll_interval)
        except (FileNotFoundError, PermissionError) as e:
            logger.warning("tail_log 读文件异常 path=%s: %s", log_file_path, e)
            yield _sse(f"[错误] 无法读取日志: {e}")
        except GeneratorExit:
            logger.debug("tail_log 客户端断开 path=%s", log_file_path)
            raise

    def _parse_time_range(self, time_range_str, start_time_str, end_time_str):
        """
        解析时间范围。返回 (start_ts, end_ts)，均为 datetime 或 None。
        - time_range: 1m | 10m | 30m | 1h，表示「最近 N」；end=now，start=now-N。
        - start_time / end_time: 自定义起止，格式 YYYY-MM-DD HH:MM:SS 或 YYYY-MM-DD。
        若 time_range 与 start/end 同时传，time_range 优先。
        """
        now = datetime.now()
        if time_range_str:
            s = (time_range_str or "").strip().lower()
            delta = None
            if s == "1m":
                delta = timedelta(minutes=1)
            elif s == "10m":
                delta = timedelta(minutes=10)
            elif s == "30m":
                delta = timedelta(minutes=30)
            elif s == "1h":
                delta = timedelta(hours=1)
            if delta is not None:
                return now - delta, now

        def _parse_dt(s):
            if not s:
                return None
            s = s.strip()
            for fmt in ("%Y-%m-%d %H:%M:%S", "%Y-%m-%d"):
                try:
                    return datetime.strptime(s, fmt)
                except ValueError:
                    continue
            return None

        start_ts = _parse_dt(start_time_str)
        end_ts = _parse_dt(end_time_str)
        if end_ts is None and start_ts is not None:
            end_ts = now
        return start_ts, end_ts

    @action(methods=["get"], detail=False, permission_classes=[AdminPermission])
    def tail_logs(self, request, *args, **kwargs):
        """
        使用 Server-Sent Events (EventSource) 实时推送日志内容，类似 Linux tail -f。
        支持按时间范围过滤，仅输出指定时间段内的日志行。
        未传 max_duration_seconds/max_lines 时默认 30 秒后结束；传 max_duration_seconds=0 表示不限制（长连）。
        查询参数:
          - filename: server.log | error.log，默认 server.log
          - from_beginning: 1 或 true 表示从文件开头读起，否则从当前末尾开始
          - poll_interval: 轮询间隔秒数，默认 0.5
          - time_range: 1m | 10m | 30m | 1h，表示最近 1 分钟 / 10 分钟 / 30 分钟 / 1 小时（优先）
          - start_time: 自定义开始时间，格式 YYYY-MM-DD HH:MM:SS 或 YYYY-MM-DD
          - end_time: 自定义结束时间，格式同上；不传则默认为当前时间
          - max_duration_seconds: 最多流式传输秒数，超时后结束；默认 30；传 0 表示不限制
          - max_lines: 最多输出日志行数，达到后结束
        """
        log_filename = request.GET.get("filename", "server.log")
        logger.info("tail_logs 请求 filename=%s from_beginning param=%s", log_filename, request.GET.get("from_beginning"))

        if log_filename not in ("server.log", "error.log"):
            logger.warning("tail_logs 无效文件名 filename=%s", log_filename)
            return ErrorResponse(msg="无效的日志文件名，仅支持 server.log 或 error.log")

        logs_dir = os.path.join(settings.BASE_DIR, "logs")
        log_file_path = os.path.join(logs_dir, log_filename)
        if not os.path.exists(log_file_path):
            logger.warning("tail_logs 文件不存在 path=%s", log_file_path)
            return ErrorResponse(msg=f"日志文件不存在: {log_filename}")

        from_beginning = request.GET.get("from_beginning", "").lower() in ("1", "true", "yes")
        try:
            poll_interval = max(0.1, min(2.0, float(request.GET.get("poll_interval", 0.5))))
        except (TypeError, ValueError):
            poll_interval = 0.5

        time_range = request.GET.get("time_range", "").strip()
        start_time = request.GET.get("start_time", "").strip()
        end_time = request.GET.get("end_time", "").strip()
        start_ts, end_ts = self._parse_time_range(time_range, start_time, end_time)
        if (start_ts or end_ts) and not from_beginning:
            from_beginning = True

        max_duration_seconds = None
        try:
            v = request.GET.get("max_duration_seconds")
            if v is not None and str(v).strip() != "":
                n = int(float(v))
                if n > 0:
                    max_duration_seconds = max(1, min(86400, n))
                # n<=0 表示显式不限制，保持 None
        except (TypeError, ValueError):
            pass
        max_lines = None
        try:
            v = request.GET.get("max_lines")
            if v is not None and v != "":
                max_lines = max(1, min(1000000, int(v)))
        except (TypeError, ValueError):
            pass
        # 未传任何限制时默认 30 秒结束，避免 curl 等客户端一直无返回
        if max_duration_seconds is None and max_lines is None:
            max_duration_seconds = 30
            # 默认限时场景下从文件开头读，先输出已有日志，否则只等新行会一直无内容
            if request.GET.get("from_beginning", "").lower() not in ("0", "false", "no"):
                from_beginning = True

        stream = self._tail_log_stream(
            log_file_path,
            from_beginning=from_beginning,
            poll_interval=poll_interval,
            start_ts=start_ts,
            end_ts=end_ts,
            max_duration_seconds=max_duration_seconds,
            max_lines=max_lines,
        )
        response = StreamingHttpResponse(stream, content_type="text/event-stream; charset=utf-8")
        response["Cache-Control"] = "no-cache"
        response["X-Accel-Buffering"] = "no"
        response["Connection"] = "keep-alive"
        return response



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : im_session.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 会话表
import logging

from django.core.cache import cache
from rest_framework import serializers

from django_base_ai.system.models import IMChatToGroupMessage, IMChatToUser, IMSession
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class IMSessionSerializer(CustomModelSerializer):
    userinfo = serializers.SerializerMethodField()
    seats = serializers.SerializerMethodField()
    session_id = serializers.SerializerMethodField()
    un_read_count = serializers.SerializerMethodField()
    last_message = serializers.SerializerMethodField()
    update_datetime = serializers.SerializerMethodField()
    create_datetime = serializers.DateTimeField(format="%Y-%m-%d %H:%M:%S", required=False, read_only=True)

    def get_userinfo(self, obj):
        from_user = obj.parent.from_user if obj.parent else obj.from_user
        return {
            "uid": from_user.id,
            "username": from_user.username,
            "email": from_user.email,
            "mobile": from_user.mobile,
            "landline": from_user.landline,
            "avatar": from_user.avatar,
            "name": from_user.name,
            "alias": from_user.alias,
            "main_department": from_user.main_department,
            "dept": from_user.dept.name if from_user.dept else "",
        }

    def get_un_read_count(self, obj):
        session_id = obj.parent if obj.parent else obj
        if obj.session_type:
            return IMChatToGroupMessage.objects.filter(
                im_chat_to_group__im_session=session_id, from_to_user=self.request.user, is_read=True
            ).count()
        return IMChatToUser.objects.filter(im_session=session_id, from_to_user=self.request.user, is_read=True).count()

    def get_last_message(self, obj):
        return cache.get(f"{obj.parent.session_id if obj.parent else obj.session_id}_{self.request.user.id}", {})

    def get_update_datetime(self, obj):
        return obj.update_datetime.strftime("%H:%M")

    def get_session_id(self, obj):
        return obj.parent.session_id if obj.parent else obj.session_id

    def get_seats(self, obj):
        if not obj.ai_desk_app_manage:
            return None
        return obj.ai_desk_app_manage.seats

    class Meta:
        model = IMSession
        fields = [
            "id",
            "session_id",
            "userinfo",
            "is_it_pinned",
            "is_claim",
            "is_colse",
            "un_read_count",
            "last_message",
            "ai_desk_app_manage",
            "seats",
            "create_datetime",
            "update_datetime",
        ]
        read_only_fields = ["id"]


class IMSessionCreateSerializer(CustomModelSerializer):
    def save(self, **kwargs):
        kwargs.update({"from_user": self.request.user})
        data = super().save(**kwargs)
        data.save()
        return data

    class Meta:
        model = IMSession
        fields = "__all__"


class IMSessionUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = IMSession
        fields = "__all__"


class IMSessionViewSet(CustomModelViewSet):
    """
    会话管理
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = IMSession.objects.all().filter(is_colse=False).order_by("-create_datetime")
    serializer_class = IMSessionSerializer
    filter_fields = ["session_id", "is_it_pinned", "is_claim", "is_colse", "ai_desk_app_manage"]
    create_serializer_class = IMSessionCreateSerializer
    update_serializer_class = IMSessionUpdateSerializer
    # extra_filter_backends = []



"""数据分析 AnalyzeView 接口单元测试。"""

import pytest
from rest_framework import status


pytestmark = [pytest.mark.django_db]

API = "/base/api/system/base_analyze"


def test_base_analyze_list_or_actions_ok(api_client):
    """GET /base_analyze/ 可能 AllowAny，检查可访问。"""
    response = api_client.get(API + "/")
    assert response.status_code in (status.HTTP_200_OK, status.HTTP_401_UNAUTHORIZED, status.HTTP_404_NOT_FOUND)


def test_base_analyze_with_auth_ok(authenticated_client):
    """GET /base_analyze/ 已认证返回 200 或 404（无 action 时）。"""
    response = authenticated_client.get(API + "/")
    assert response.status_code in (status.HTTP_200_OK, status.HTTP_404_NOT_FOUND)



#!/usr/bin/env python
"""
@Project ：django_base_ai
@File    ：execute_task.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：定时任务执行与 HTTP 请求发送
"""

import time
from datetime import datetime

import httpx
from django_celery_beat.models import PeriodicTask

from django_base_ai.system.models import BaseTask, TaskRunHistory


class ExecuteRequestTask:
    def __init__(self, nid, pid=None):
        self.nid = nid
        self.pid = pid

    def request(self, client, infos):
        reqmethod = infos.get("reqmethod", "GET")
        url = infos.get("url", "")
        headers = infos.get("headers", {})
        payload = infos.get("payload", {})
        client.verify = True if "https://" in url else False
        try:
            resp = client.request(reqmethod, url, headers=headers, data=payload)
            resp_result = resp.text if resp.status_code == 200 else str(resp.extensions)
            return resp_result, resp.status_code
        except Exception as e:
            return e.args, 500

    def execute_task(self):
        task_management = BaseTask.objects.filter(id=self.nid, task_status=True).first()
        if task_management is None:
            return f"error={self.nid}"
        request_config = task_management.request_config
        limits = httpx.Limits(max_keepalive_connections=0, max_connections=100)
        with httpx.Client(limits=limits, timeout=5) as client:
            task_run_history = []
            for infos in request_config:
                start_time = datetime.now()
                url_name = infos.get("name", "")
                infos = {
                    "reqmethod": infos.get("reqmethod", "GET"),
                    "headers": infos.get("reqheaders", {}),
                    "url": infos.get("url", "#"),
                    "payload": infos.get("payload", {}),
                }
                resp_result, status_code = self.request(client, infos)
                if self.pid:
                    PeriodicTask.objects.filter(id=self.pid).update(last_run_at=datetime.now())
                task_run_history.append(
                    TaskRunHistory(
                        base_task_id=self.nid,
                        url_name=url_name,
                        start_time=start_time,
                        end_time=datetime.now(),
                        request_config=infos,
                        status_code=status_code,
                        run_result=resp_result,
                        periodic_task_id=self.pid,
                    )
                )
            if len(task_run_history):
                TaskRunHistory.objects.bulk_create(task_run_history)

    def run(self):
        start_time = time.time()
        self.execute_task()
        end_time = time.time()
        return end_time - start_time



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : folder_list.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 文件夹列表与上传管理接口
import logging

from django_base_ai.system.models import FolderList
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class FolderSerializer(CustomModelSerializer):
    class Meta:
        model = FolderList
        fields = "__all__"


class FolderViewSet(CustomModelViewSet):
    """
    文件夹管理接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = FolderList.objects.all()
    serializer_class = FolderSerializer
    filter_fields = [
        "name",
    ]
    permission_classes = []



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : menu_button.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 菜单按钮（权限点）CRUD。
import logging

from django.db.models import CharField, Value
from django.db.models.functions import Concat
from rest_framework.decorators import action

from django_base_ai.system.models import Menu, MenuButton
from django_base_ai.utils.json_response import DetailResponse
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class MenuButtonSerializer(CustomModelSerializer):
    """
    菜单按钮-序列化器
    """

    class Meta:
        model = MenuButton
        fields = ["id", "name", "value", "api", "method", "menu"]
        read_only_fields = ["id"]


class MenuButtonInitSerializer(CustomModelSerializer):
    """
    初始化菜单按钮-序列化器
    """

    class Meta:
        model = MenuButton
        fields = ["id", "name", "value", "api", "method", "menu"]
        read_only_fields = ["id"]


class MenuButtonCreateUpdateSerializer(CustomModelSerializer):
    """
    初始化菜单按钮-序列化器
    """

    class Meta:
        model = MenuButton
        fields = "__all__"
        read_only_fields = ["id"]


class MenuButtonViewSet(CustomModelViewSet):
    """
    菜单按钮接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = MenuButton.objects.all()
    serializer_class = MenuButtonSerializer
    create_serializer_class = MenuButtonCreateUpdateSerializer
    update_serializer_class = MenuButtonCreateUpdateSerializer
    # extra_filter_backends = []

    @action(methods=["GET"], detail=False, permission_classes=[])
    def get_btn_permission(self, request):
        """
        获取当前用户的按钮权限
        """
        user = request.user
        if not user.is_superuser:
            menu_ids = user.role.values_list("menu__id", flat=True)
        else:
            menu_ids = Menu.objects.filter(status=1)
        queryset = (
            MenuButton.objects.filter(menu__in=menu_ids)
            .annotate(permission=Concat("menu__web_path", Value(":"), "value", output_field=CharField()))
            .values_list("permission", flat=True)
        )
        return DetailResponse(data=queryset)



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : im_chat_to_group_message.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 群聊发送消息

import logging
from datetime import datetime

import django_filters
from django.core.cache import cache
from django_filters.rest_framework import FilterSet
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated

from django_base_ai.system.models import IMChatToGroup, IMChatToGroupMessage
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet
from django_base_ai.websocket.websocket_config import websocket_push

logger = logging.getLogger(__name__)


class IMChatToGroupMessageSerializer(CustomModelSerializer):
    class Meta:
        model = IMChatToGroupMessage
        fields = "__all__"
        read_only_fields = ["id"]


class IMChatToGroupMessageCreateUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = IMChatToGroupMessage
        fields = "__all__"


class FilterIMChatToGroupMessage(FilterSet):
    im_session = django_filters.CharFilter(field_name="im_chat_to_group__session_id", method="filter_im_session")

    def filter_im_session(self, queryset, name, value):
        nid = self.request.GET.get("im_session")
        if IMChatToGroupMessage.objects.filter(
            from_to_user=self.request.user, im_chat_to_group__im_session__session_id=nid, is_read=True
        ).count():
            # 出于性能考虑先求总数
            IMChatToGroupMessage.objects.filter(
                from_to_user=self.request.user, im_chat_to_group__im_session__session_id=nid, is_read=True
            ).update(is_read=False)
        return queryset.filter(from_to_user=self.request.user, im_chat_to_group__im_session__session_id=nid)

    class Meta:
        model = IMChatToGroupMessage
        fields = ["im_session"]


class IMChatToGroupMessageViewSet(CustomModelViewSet):
    """
    群聊内容
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = IMChatToGroupMessage.objects.all().order_by("-id")
    serializer_class = IMChatToGroupMessageSerializer
    # extra_filter_backends = []
    filter_class = FilterIMChatToGroupMessage

    def create(self, request, *args, **kwargs):
        current_user = request.user
        copy_data = request.data
        im_session = copy_data.get("im_session", "")
        content = copy_data.get("content", {})
        # True 只发送自己
        send_self = copy_data.get("sendSelf", False)
        if im_session == "":
            return ErrorResponse(msg="参数不合法")
        # from_many_to_user__id__in=[current_user.id] 报错
        im_chat_to_group = IMChatToGroup.objects.filter(im_session__session_id=im_session).first()
        if not im_chat_to_group:
            return ErrorResponse(msg="群组不存在或您无权限访问")
        if send_self:
            from_many_to_user = [current_user.id]
        else:
            from_many_to_user = im_chat_to_group.from_many_to_user.all().values_list("id", flat=True)
            print("from_many_to_user=", from_many_to_user)
            copy_data.pop("im_session")
        im_chat_to_group_message_list = []
        for user in from_many_to_user:
            im_chat_to_group_message_list.append(
                IMChatToGroupMessage(
                    im_chat_to_group_id=im_chat_to_group.id,
                    from_user_id=current_user.id,
                    is_read=False if user == current_user.id else True,
                    from_to_user_id=user,
                    content=content,
                )
            )
            print(f"{user}_{im_chat_to_group.group_ext_info.get(str(current_user.id), 0)}发送成功")
            if int(user) != current_user.id or send_self:
                websocket_push(
                    f"{user}_{im_chat_to_group.group_ext_info.get(str(user), 0)}",
                    message={
                        "uid": current_user.id,
                        "username": current_user.username,
                        "im_session": im_session,
                        "email": current_user.email,
                        "mobile": current_user.mobile,
                        "landline": current_user.landline,
                        "avatar": current_user.avatar,
                        "name": current_user.name,
                        "update_datetime": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                        "create_datetime": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                        "data": copy_data,
                    },
                )
            # redis 记录最后一条消息
            cache.set(f"{im_session}_{user}", content, 3000)
        IMChatToGroupMessage.objects.bulk_create(im_chat_to_group_message_list)
        return DetailResponse(msg="发送成功")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def update_read(self, request, *args, **kwargs):
        nid = self.request.GET.get("im_session")
        IMChatToGroupMessage.objects.filter(
            from_to_user=self.request.user, im_chat_to_group__im_session__session_id=nid, is_read=True
        ).update(is_read=False)
        return DetailResponse(msg="完成")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Django 通用底座默认配置：DRF、JWT、分页、认证、异常处理、日志、API 记录等。
主工程使用方式：
  from django_base_ai.settings import *
  # 再覆盖 DATABASES, INSTALLED_APPS 等（可选覆盖 SECRET_KEY）
SECRET_KEY 由底座从环境变量 SECRET_KEY 读取，多应用共用同一底座时使用同一密钥即可共用会话/JWT 等能力，新建项目仅需处理业务逻辑、无需单独配置密钥；若需项目独立密钥可在引入后覆盖 SECRET_KEY。
"""

"""
Django settings for application project.

Generated by 'django-admin startproject' using Django 3.2.3.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.2/ref/settings/
"""

import os
from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

# 底座统一 SECRET_KEY：从环境变量读取，同一底座上的应用使用相同 SECRET_KEY 可共用会话、JWT、签名等
# 子项目不覆盖即共用；生产环境请在部署层设置环境变量 SECRET_KEY
SECRET_KEY = os.environ.get(
    "SECRET_KEY",
    "django-insecure-dev-change-me-when-using-base-default",
)

API_LOG_ENABLE = True
APPID = locals().get("APPID", "未知")
# API_LOG_METHODS = 'ALL'
API_LOG_METHODS = ["POST", "UPDATE", "DELETE", "PUT"]  # ['POST', 'DELETE']
# API_MODEL_MAP、API_EXEMPT_MAP 见下方「其他配置」节，主工程可覆盖
PATH_MAPPING = locals().get(
    "PATH_MAPPING",
    {
        "/healthy/": "健康检查",
        "/token/refresh/": "刷新Token",
    },
)
# Password validation
# https://docs.djangoproject.com/en/3.2/ref/settings/#auth-password-validators


# Internationalization
# https://docs.djangoproject.com/en/3.2/topics/i18n/

LANGUAGE_CODE = "zh-hans"

TIME_ZONE = "Asia/Shanghai"

USE_I18N = True

USE_L10N = True

USE_TZ = False

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.2/howto/static-files/

STATIC_URL = "/static/"
static_path = os.path.join(BASE_DIR, "static")
if not os.path.exists(static_path):
    os.makedirs(static_path)
# # 设置django的静态文件目录
STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "static"),
]

MEDIA_ROOT = "media"  # 项目下的目录
MEDIA_URL = "media/"  # 跟STATIC_URL类似，指定用户可以通过这个url找到文件

# 收集静态文件，必须将 MEDIA_ROOT,STATICFILES_DIRS先注释
# python manage.py collectstatic
# STATIC_ROOT=os.path.join(BASE_DIR,'static')

# ================================================= #
# ******************* 跨域的配置 ******************* #
# ================================================= #

# 全部允许配置
CORS_ORIGIN_ALLOW_ALL = True
# 允许cookie
CORS_ALLOW_CREDENTIALS = True  # 指明在跨域访问中，后端是否支持对cookie的操作

# ================================================= #
# ********************* channels配置 ******************* #
# ================================================= #
# 由主工程覆盖：ASGI_APPLICATION = 'YourProject.asgi.application'
ASGI_APPLICATION = locals().get("ASGI_APPLICATION", None)
REDIS_MODE = locals().get("REDIS_MODE", 0)
REDIS_HOST = locals().get("REDIS_HOST", "")
REDIS_URL = locals().get("REDIS_URL", "")
REDIS_URLS = locals().get("REDIS_HOSTS", [])

# ================================================= #
# *************** REST_FRAMEWORK配置 *************** #
# ================================================= #

REST_FRAMEWORK = {
    "DATETIME_FORMAT": "%Y-%m-%d %H:%M:%S",  # 日期时间格式配置
    "DATE_FORMAT": "%Y-%m-%d",
    "DEFAULT_FILTER_BACKENDS": (
        # 'django_filters.rest_framework.DjangoFilterBackend',
        "django_base_ai.utils.filters.CustomDjangoFilterBackend",
        "rest_framework.filters.SearchFilter",
        "rest_framework.filters.OrderingFilter",
    ),
    "DEFAULT_PAGINATION_CLASS": "django_base_ai.utils.pagination.CustomPagination",  # 自定义分页
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "django_base_ai.utils.authentication.UserAuthentication",  # 自定义的认证类-用于处理get请求access_token发送的凭证
        "rest_framework_simplejwt.authentication.JWTAuthentication",
        "rest_framework.authentication.SessionAuthentication",
    ),
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.IsAuthenticated",  # 只有经过身份认证确定用户身份才能访问
        # 'rest_framework.permissions.IsAdminUser', # is_staff=True才能访问 —— 管理员(员工)权限
        # 'rest_framework.permissions.AllowAny', # 允许所有
        # 'rest_framework.permissions.IsAuthenticatedOrReadOnly', # 有身份 或者 只读访问(self.list,self.retrieve)
    ],
    "EXCEPTION_HANDLER": "django_base_ai.utils.exception.custom_exception_handler",  # 自定义的异常处理
    "DEFAULT_RENDERER_CLASSES": ("rest_framework.renderers.JSONRenderer",),  # 关闭 API 调试界面，主工程可覆盖
}
# ================================================= #
# ******************** 登录方式配置 ******************** #
# ================================================= #

AUTHENTICATION_BACKENDS = ["django_base_ai.utils.backends.CustomBackend"]
# ================================================= #
# ****************** simplejwt配置 ***************** #
# ================================================= #
from datetime import timedelta

SIMPLE_JWT = {
    # token有效时长
    "ACCESS_TOKEN_LIFETIME": timedelta(minutes=15),
    # token刷新后的有效时间
    "REFRESH_TOKEN_LIFETIME": timedelta(minutes=30),
    # 设置前缀
    "AUTH_HEADER_TYPES": ("JWT",),
    "ROTATE_REFRESH_TOKENS": True,
    "BLACKLIST_AFTER_ROTATION": True,
    "AUTH_TOKEN_CLASSES": ("rest_framework_simplejwt.tokens.AccessToken",),
}

# ================================================= #
# **************** 验证码配置  ******************* #
# ================================================= #
CAPTCHA_IMAGE_SIZE = (160, 60)  # 设置 captcha 图片大小
CAPTCHA_LENGTH = 4  # 字符个数
CAPTCHA_TIMEOUT = 1  # 超时(minutes)
CAPTCHA_OUTPUT_FORMAT = "%(image)s %(text_field)s %(hidden_field)s "
CAPTCHA_FONT_SIZE = 40  # 字体大小
CAPTCHA_FOREGROUND_COLOR = "#64DAAA"  # 前景色
CAPTCHA_BACKGROUND_COLOR = "#F5F7F4"  # 背景色
CAPTCHA_NOISE_FUNCTIONS = (
    "captcha.helpers.noise_arcs",  # 线
    # "captcha.helpers.noise_dots",  # 点
)
# CAPTCHA_CHALLENGE_FUNCT = 'captcha.helpers.random_char_challenge' #字母验证码
CAPTCHA_CHALLENGE_FUNCT = "captcha.helpers.math_challenge"  # 加减乘除验证码

# ================================================= #
# ******************** 其他配置 ******************** #
# ================================================= #

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
# 是否启动API日志记录
API_LOG_ENABLE = locals().get("API_LOG_ENABLE", True)
# API 日志记录的请求方式
API_LOG_METHODS = locals().get("API_LOG_METHODS", ["POST", "UPDATE", "DELETE", "PUT"])
# API_LOG_METHODS = 'ALL' # ['POST', 'DELETE']
# 在操作日志中详细记录的请求模块映射
API_MODEL_MAP = locals().get(
    "API_MODEL_MAP",
    {
        "/token/": "登录模块",
        "/base/api/login/": "登录模块",
        "/base/api/logout/": "登录模块",
    },
)
API_EXEMPT_MAP = locals().get(
    "API_EXEMPT_MAP",
    {
        "/healthy/": "健康检查",
        "/token/refresh/": "刷新Token",
    },
)
# 是否启动 License 验证（关闭则不做 License 校验）
LICENSE_VERIFY_ENABLE = locals().get("LICENSE_VERIFY_ENABLE", False)
# License 验证排除路径（请求 path 包含以下任一则跳过校验）
LICENSE_VERIFY_EXCLUDE = locals().get(
    "LICENSE_VERIFY_EXCLUDE",
    [
        "/healthy",
        "/readiness",
        "/static",
        "/media",
        "license/verify",
    ],
)
DJANGO_CELERY_BEAT_TZ_AWARE = False
CELERY_TIMEZONE = "Asia/Shanghai"  # celery 时区问题
# 静态页面压缩
STATICFILES_STORAGE = "whitenoise.storage.CompressedStaticFilesStorage"

ALL_MODELS_OBJECTS = []  # 所有app models 对象

# 初始化需要执行的列表，用来初始化后执行
INITIALIZE_LIST = []
INITIALIZE_RESET_LIST = []
# 表前缀（主工程可覆盖，默认空）
TABLE_PREFIX = locals().get("TABLE_PREFIX", "")
# 系统配置
SYSTEM_CONFIG = {}
# 字典配置
DICTIONARY_CONFIG = {}
# 系统配置存放位置：redis/memory(默认)
DISPATCH_DB_TYPE = "memory"

# ================================================= #
# ******************** 插件配置 ******************** #
# ================================================= #
# 插件 urlpatterns
PLUGINS_URL_PATTERNS = []

SECURE_CROSS_ORIGIN_OPENER_POLICY = "None"
CELERY_BROKER_CONNECTION_RETRY_ON_STARTUP = True

# # ================================================= #
# # ********************* 日志配置 ******************* #
# # ================================================= #
# # log 配置部分BEGIN #
SERVER_LOGS_FILE = os.path.join(BASE_DIR, "logs", "server.log")
ERROR_LOGS_FILE = os.path.join(BASE_DIR, "logs", "error.log")
LOGS_FILE = os.path.join(BASE_DIR, "logs")
if not os.path.exists(os.path.join(BASE_DIR, "logs")):
    os.makedirs(os.path.join(BASE_DIR, "logs"))

# 格式: 2025-12-30T10:58:00 [INFO] [app.xx.xxx.xxx] -- 登录成功
# 格式: 日期时间 [级别] [模块名] -- 信息
STANDARD_LOG_FORMAT = "%(asctime)s [%(levelname)s] [%(name)s] -- %(message)s"
CONSOLE_LOG_FORMAT = "%(asctime)s [%(levelname)s] [%(name)s] -- %(message)s"
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "standard": {
            "format": STANDARD_LOG_FORMAT,
            "datefmt": "%Y-%m-%dT%H:%M:%S",
        },
        "console": {
            "format": CONSOLE_LOG_FORMAT,
            "datefmt": "%Y-%m-%dT%H:%M:%S",
        },
        "file": {
            "format": CONSOLE_LOG_FORMAT,
            "datefmt": "%Y-%m-%dT%H:%M:%S",
        },
    },
    "handlers": {
        "file": {
            "level": "INFO",
            "class": "logging.handlers.RotatingFileHandler",
            "filename": SERVER_LOGS_FILE,
            "maxBytes": 1024 * 1024 * 100,  # 100 MB
            "backupCount": 5,  # 最多备份5个
            "formatter": "standard",
            "encoding": "utf-8",
        },
        "error": {
            "level": "ERROR",
            "class": "logging.handlers.RotatingFileHandler",
            "filename": ERROR_LOGS_FILE,
            "maxBytes": 1024 * 1024 * 100,  # 100 MB
            "backupCount": 3,  # 最多备份3个
            "formatter": "standard",
            "encoding": "utf-8",
        },
        "console": {
            "level": "INFO",
            "class": "logging.StreamHandler",
            "formatter": "console",
        },
    },
    "loggers": {
        "": {
            "handlers": ["console", "error", "file"],
            "level": "INFO",
        },
        "django": {
            "handlers": ["console", "error", "file"],
            "level": "INFO",
            "propagate": False,
        },
        "django.db.backends": {"handlers": ["console", "error", "file"], "propagate": False, "level": "INFO"},
        "uvicorn.error": {
            "level": "INFO",
            "handlers": ["console", "error", "file"],
        },
        "uvicorn.access": {"handlers": ["console", "error", "file"], "level": "INFO"},
    },
}
# 任务状态
TASK_STATUS = {"PENDING": "待处理", "STARTED": "已开始", "RETRY": "重试", "FAILURE": "失败", "SUCCESS": "成功"}

# XSS
ALLOWED_TAGS = [
    "a",
    "font",
    "abbr",
    "address",
    "area",
    "article",
    "aside",
    "audio",
    "b",
    "base",
    "bdi",
    "bdo",
    "big",
    "blockquote",
    "body",
    "br",
    "button",
    "canvas",
    "caption",
    "cite",
    "code",
    "col",
    "colgroup",
    "data",
    "datalist",
    "dd",
    "del",
    "details",
    "dfn",
    "dialog",
    "div",
    "dl",
    "dt",
    "em",
    "embed",
    "fieldset",
    "figcaption",
    "figure",
    "footer",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "head",
    "header",
    "hr",
    "html",
    "i",
    "img",
    "input",
    "ins",
    "kbd",
    "label",
    "legend",
    "li",
    "link",
    "main",
    "map",
    "mark",
    "meta",
    "meter",
    "nav",
    "noscript",
    "object",
    "ol",
    "optgroup",
    "option",
    "output",
    "p",
    "param",
    "picture",
    "pre",
    "progress",
    "q",
    "rp",
    "rt",
    "ruby",
    "s",
    "samp",
    "script",
    "section",
    "select",
    "small",
    "source",
    "span",
    "strong",
    "style",
    "sub",
    "summary",
    "sup",
    "svg",
    "table",
    "tbody",
    "td",
    "template",
    "textarea",
    "tfoot",
    "th",
    "thead",
    "time",
    "title",
    "tr",
    "track",
    "u",
    "ul",
    "video",
    "wbr",
]  # 允许的标签
ALLOWED_ATTRS = {
    "*": ["style"],  # 所有标签允许 style 属性
    "a": ["href", "title"],  # a 标签允许 href 和 title 属性
    "img": ["src", "alt"],  # img 标签允许 src 和 alt 属性
}
ALLOWED_STYLES = [
    "color",
    "font-family",
    "font-style",
    "text-align",
    "text-decoration",
    "text-indent",
    "background-color",
    "line-height",
    "letter-spacing",
    "word-spacing",
]

"""
[ 'color', 'font-family', 'font-size', 'font-style', 'font-weight', 'text-align', 'text-decoration', 'text-indent',
                   'background-color', 'background-image', 'background-repeat', 'background-size', 'border', 'border-bottom',
                   'border-left', 'border-radius', 'border-right', 'border-top', 'margin', 'margin-bottom', 'margin-left',
                   'margin-right', 'margin-top', 'padding', 'padding-bottom', 'padding-left', 'padding-right', 'padding-top',
                   'line-height', 'letter-spacing', 'word-spacing']
"""



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : message_center_target_user.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 消息中心目标用户（接收人）CRUD 与已读等。
import logging

from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated

from django_base_ai.system.models import MessageCenterTargetUser
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse, SuccessResponse
from django_base_ai.utils.permission import OpenApiPermission
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class MessageCenterTargetUserSerializer(CustomModelSerializer):
    u_id = serializers.IntegerField(read_only=True, source="users.id")
    name = serializers.StringRelatedField(read_only=True, source="users.name")
    username = serializers.StringRelatedField(read_only=True, source="users.username")
    title = serializers.StringRelatedField(read_only=True, source="messagecenter.title")
    short_content = serializers.StringRelatedField(read_only=True, source="messagecenter.short_content")
    thumbnail = serializers.StringRelatedField(read_only=True, source="messagecenter.thumbnail")
    message_type = serializers.IntegerField(read_only=True, source="messagecenter.message_type")
    content = serializers.StringRelatedField(read_only=True, source="messagecenter.content")
    url = serializers.StringRelatedField(read_only=True, source="messagecenter.url")
    target_type = serializers.IntegerField(read_only=True, source="messagecenter.target_type")
    is_withdraw = serializers.BooleanField(read_only=True, source="messagecenter.is_withdraw")
    ext_kwargs = serializers.JSONField(read_only=True, source="messagecenter.ext_kwargs")
    description = serializers.JSONField(read_only=True, source="messagecenter.description")

    class Meta:
        model = MessageCenterTargetUser
        fields = "__all__"
        read_only_fields = ["id"]
        # depth = 1


class ExportUserProfileSerializer(CustomModelSerializer):
    """
    用户导出 序列化器
    """

    is_read = serializers.SerializerMethodField(read_only=True)

    def get_is_read(self, instance):
        return "已读" if instance.is_read else "未读"

    class Meta:
        model = MessageCenterTargetUser
        fields = ("users__username", "is_read", "create_datetime", "update_datetime")


class MessageCenterTargetUserViewSet(CustomModelViewSet):
    """
    消息中心用户列表接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = MessageCenterTargetUser.objects.order_by("-id")
    serializer_class = MessageCenterTargetUserSerializer
    filter_fields = ["messagecenter", "$users__name"]
    search_fields = ["messagecenter__title", "messagecenter__short_content"]
    # 导出
    export_field_label = {
        "users__username": "用户账号",
        "is_read": "是否已读",
        "create_datetime": "创建时间",
        "update_datetime": "修改时间",
    }
    export_serializer_class = ExportUserProfileSerializer

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def get_first_msg(self, request):
        """
        获取最新的一条消息
        """
        self_user_id = self.request.user.id
        instance = (
            MessageCenterTargetUser.objects.filter(users__id=self_user_id, is_read=False)
            .order_by("-create_datetime")
            .first()
        )
        if not instance:
            return DetailResponse(msg="没有未读消息")
        serializer = self.get_serializer(instance)
        return DetailResponse(msg="已读", data=serializer.data)

    @action(methods=["GET"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def open_api_message_user_list(self, request):
        nid = request.GET.get("nid", None)
        if not nid:
            return ErrorResponse("消息nid参数错误")
        queryset = MessageCenterTargetUser.objects.filter(messagecenter__id=nid).all()
        serializer = self.get_serializer(queryset, many=True, request=request)
        return SuccessResponse(data=serializer.data, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def my_message(self, request, *args, **kwargs):
        """
        获取我的消息
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        queryset = MessageCenterTargetUser.objects.filter(users__id=self.request.user.id).order_by("-create_datetime")
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True, request=request)
            return self.get_paginated_response(serializer.data)
        serializer = self.get_serializer(queryset, many=True, request=request)
        return SuccessResponse(data=serializer.data, msg="获取成功")

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def click_one_read(self, request, *args, **kwargs):
        mid = request.GET.get("mid", None)
        if not mid:
            return ErrorResponse("消息ID参数错误")
        instance = MessageCenterTargetUser.objects.filter(id=mid, users__id=request.user.id).first()
        instance.is_read = True
        instance.save(update_fields=["is_read"])
        serializer = self.get_serializer(instance)
        return DetailResponse(msg="已读", data=serializer.data)



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : system_config.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 系统配置键值、树形与表关联、初始化。
import logging
import os

import django_filters
from django.core.cache import cache
from django.db.models import Q
from django_filters.rest_framework import BooleanFilter
from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.permissions import IsAdminUser
from rest_framework.views import APIView

from django_base_ai import dispatch
from django_base_ai.system.models import SystemConfig
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse, SuccessResponse
from django_base_ai.utils.models import get_all_models_objects
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.validator import CustomValidationError
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class SystemConfigCreateUpdateSerializer(CustomModelSerializer):
    """
    系统配置-新增时使用-序列化器
    """

    form_item_type_label = serializers.CharField(source="get_form_item_type_display", read_only=True)

    class Meta:
        model = SystemConfig
        fields = "__all__"
        read_only_fields = ["id"]

    def validate_key(self, value):
        """
        验证key是否允许重复
        parent为空时不允许重复,反之允许
        """
        instance = SystemConfig.objects.filter(key=value, parent__isnull=True).exists()
        if instance:
            raise CustomValidationError("已存在相同变量名")
        return value


class SystemConfigInitSerializer(CustomModelSerializer):
    """
    初始化获取数信息(用于生成初始化json文件)
    """

    children = serializers.SerializerMethodField()

    def get_children(self, obj: SystemConfig):
        data = []
        instance = SystemConfig.objects.filter(parent_id=obj.id)
        if instance:
            serializer = SystemConfigInitSerializer(instance=instance, many=True)
            data = serializer.data
        return data

    def save(self, **kwargs):
        instance = super().save(**kwargs)
        children = self.initial_data.get("children")
        # 菜单表
        if children:
            for data in children:
                data["parent"] = instance.id
                filter_data = {"key": data["key"], "parent": data["parent"]}
                instance_obj = SystemConfig.objects.filter(**filter_data).first()
                if instance_obj and not self.initial_data.get("reset"):
                    continue
                serializer = SystemConfigInitSerializer(instance_obj, data=data, request=self.request)
                serializer.is_valid(raise_exception=True)
                serializer.save()
        return instance

    class Meta:
        model = SystemConfig
        fields = [
            "parent",
            "title",
            "key",
            "value",
            "sort",
            "status",
            "data_options",
            "form_item_type",
            "rule",
            "placeholder",
            "setting",
            "creator",
            "dept_belong_id",
            "children",
        ]
        read_only_fields = ["id"]
        extra_kwargs = {"creator": {"write_only": True}, "dept_belong_id": {"write_only": True}}


class SystemConfigSerializer(CustomModelSerializer):
    """
    系统配置-序列化器
    """

    form_item_type_label = serializers.CharField(source="get_form_item_type_display", read_only=True)

    class Meta:
        model = SystemConfig
        fields = "__all__"
        read_only_fields = ["id"]


class SystemConfigChinldernSerializer(CustomModelSerializer):
    """
    系统配置子级-序列化器
    """

    chinldern = serializers.SerializerMethodField()
    form_item_type_label = serializers.CharField(source="get_form_item_type_display", read_only=True)

    def get_chinldern(self, instance):
        queryset = SystemConfig.objects.filter(parent=instance)
        serializer = SystemConfigSerializer(queryset, many=True)
        return serializer.data

    class Meta:
        model = SystemConfig
        fields = "__all__"
        read_only_fields = ["id"]


class SystemConfigListSerializer(CustomModelSerializer):
    """
    系统配置下模块的保存-序列化器
    """

    def update(self, instance, validated_data):
        instance_mapping = {obj.id: obj for obj in instance}
        data_mapping = {item["id"]: item for item in validated_data}
        for obj_id, data in data_mapping.items():
            instance_obj = instance_mapping.get(obj_id, None)
            if instance_obj is None:
                return SystemConfig.objects.create(**data)
            else:
                return instance_obj.objects.update(**data)

    class Meta:
        model = SystemConfig
        fields = "__all__"
        read_only_fields = ["id"]


class SystemConfigSaveSerializer(serializers.Serializer):
    class Meta:
        read_only_fields = ["id"]
        list_serializer_class = SystemConfigListSerializer


class SystemConfigFilter(django_filters.rest_framework.FilterSet):
    """
    过滤器
    """

    parent__isnull = BooleanFilter(field_name="parent", lookup_expr="isnull")

    class Meta:
        model = SystemConfig
        fields = ["id", "parent", "status", "parent__isnull"]


class SystemConfigViewSet(CustomModelViewSet):
    """
    系统配置接口
    """

    queryset = SystemConfig.objects.order_by("sort", "create_datetime")
    serializer_class = SystemConfigChinldernSerializer
    create_serializer_class = SystemConfigCreateUpdateSerializer
    update_serializer_class = SystemConfigCreateUpdateSerializer
    retrieve_serializer_class = SystemConfigChinldernSerializer
    # filter_fields = ['id','parent']
    filter_class = SystemConfigFilter

    def save_content(self, request):
        body = request.data
        data_mapping = {item["id"]: item for item in body}
        for obj_id, data in data_mapping.items():
            instance_obj = SystemConfig.objects.filter(id=obj_id).first()
            if instance_obj is None:
                # return SystemConfig.objects.create(**data)
                serializer = SystemConfigCreateUpdateSerializer(data=data)
            else:
                serializer = SystemConfigCreateUpdateSerializer(instance_obj, data=data)
            if serializer.is_valid(raise_exception=True):
                serializer.save()
        return DetailResponse(msg="保存成功")

    def get_association_table(self, request):
        """
        获取所有的model及字段信息
        """
        res = [ele.get("table") for ele in get_all_models_objects().values()]
        return DetailResponse(msg="获取成功", data=res)

    def get_table_data(self, request, pk):
        """
        动态获取关联表的数据
        """
        instance = SystemConfig.objects.filter(id=pk).first()
        if instance is None:
            return ErrorResponse(msg="查询出错了~")
        setting = instance.setting
        if setting is None:
            return ErrorResponse(msg="查询出错了~")
        table = setting.get("table")  # 获取model名
        model = get_all_models_objects(table).get("object", {})
        # 自己判断一下不存在
        queryset = model.objects.values()
        body = request.query_params
        search_value = body.get("search", None)
        if search_value:
            search_fields = setting.get("searchField")
            filters = Q()
            filters.connector = "OR"
            for item in search_fields:
                filed = "{}__icontains".format(item.get("field"))
                filters.children.append((filed, search_value))
            queryset = model.objects.filter(filters).values()
        page = self.paginate_queryset(queryset)
        if page is not None:
            return self.get_paginated_response(queryset)
        return SuccessResponse(msg="获取成功", data=queryset, total=len(queryset))

    def get_relation_info(self, request):
        """
        查询关联的模板信息
        """
        body = request.query_params
        var_name = body.get("varName", None)
        table = body.get("table", None)
        instance = SystemConfig.objects.filter(key=var_name, setting__table=table).first()
        if instance is None:
            return ErrorResponse(msg="未获取到关联信息")
        relation_id = body.get("relationIds", None)
        relation_ids = []
        if relation_id is None:
            return ErrorResponse(msg="未获取到关联信息")
        if instance.form_item_type in [13]:
            relation_ids = [relation_id]
        elif instance.form_item_type in [14]:
            relation_ids = relation_id.split(",")
        queryset = SystemConfig.objects.filter(value__in=relation_ids).first()
        if queryset is None:
            return ErrorResponse(msg="未获取到关联信息")
        serializer = SystemConfigChinldernSerializer(queryset.parent)
        return DetailResponse(msg="查询成功", data=serializer.data)

    # 项目安全手动检查
    @action(methods=["get"], detail=False, permission_classes=[IsAdminUser])
    def check(self, request):
        from django_base_ai.utils.safety_check import SafetyCheck

        now_scan = int(request.GET.get("scan", 0))
        if not now_scan <= 0 or not cache.has_key("checkpkg"):
            SafetyCheck().check()
        safety_info = cache.get("checkpkg", {})
        data = {
            "timestamp": safety_info.get("timestamp", 0),
            "pkg_count": safety_info.get("pkg_count", 0),
            "vulnerabilities_count": safety_info.get("vulnerabilities_count", 0),
            "vulnerabilities": safety_info.get("vulnerabilities", []),
        }
        return DetailResponse(msg="完成", data=data)

    @action(methods=["get"], detail=False, permission_classes=[IsAdminUser])
    def vulnerabilities_count(self, request):
        # 漏洞包总数
        return DetailResponse(cache.get("vulnerabilities_count", 0))


class InitSettingsViewSet(APIView):
    """
    获取初始化配置
    """

    authentication_classes = []
    permission_classes = []

    def filter_system_config_values(self, data: dict):
        """
        过滤系统初始化配置
        :param data:
        :return:
        """
        if not self.request.query_params.get("key", ""):
            return data
        new_data = {}
        for key in self.request.query_params.get("key", "").split("|"):
            if key:
                new_data.update(**dict(filter(lambda x: x[0].startswith(key), data.items())))
        return new_data

    def get(self, request):
        data = dispatch.get_system_config()
        if not data:
            dispatch.refresh_system_config()
            data = dispatch.get_system_config()
        # 不返回后端专用配置
        backend_config = [
            f"{ele.get('parent__key')}.{ele.get('key')}"
            for ele in SystemConfig.objects.filter(status=False, parent_id__isnull=False).values("parent__key", "key")
        ]
        data = dict(filter(lambda x: x[0] not in backend_config, data.items()))
        # 系统默认初始化
        if data is None or len(data) == 0:
            os.system("python3 manage.py init -y")
            dispatch.refresh_system_config()
        return DetailResponse(data=data)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import operator
import re
import time
from collections import OrderedDict
from functools import reduce

from django import forms
from django.db import models
from django.db.models import F, Q
from django.db.models.constants import LOOKUP_SEP
from django_filters import utils
from django_filters.conf import settings
from django_filters.constants import ALL_FIELDS
from django_filters.filters import BooleanFilter, CharFilter
from django_filters.filterset import FilterSet, FilterSetMetaclass
from django_filters.rest_framework import DjangoFilterBackend
from django_filters.utils import get_model_field
from rest_framework.filters import BaseFilterBackend

from django_base_ai.system.models import ApiWhiteList, Dept


def get_dept(dept_id: int, dept_all_list=None, dept_list=None):
    """
    递归获取部门的所有下级部门
    :param dept_id: 需要获取的部门id
    :param dept_all_list: 所有部门列表
    :param dept_list: 递归部门list
    :return:
    """
    if not dept_all_list:
        dept_all_list = Dept.objects.all().values("id", "parent")
    if dept_list is None:
        dept_list = [dept_id]
    for ele in dept_all_list:
        if ele.get("parent") == dept_id:
            dept_list.append(ele.get("id"))
            get_dept(ele.get("id"), dept_all_list, dept_list)
    return list(set(dept_list))


class DataLevelPermissionsFilter(BaseFilterBackend):
    """
    数据 级权限过滤器
    0. 获取用户的部门id，没有部门则返回空
    1. 判断过滤的数据是否有创建人所在部门 "creator" 字段,没有则返回全部
    2. 如果用户没有关联角色则返回本部门数据
    3. 根据角色的最大权限进行数据过滤(会有多个角色，进行去重取最大权限)
    4. 判断用户是否为超级管理员角色/如果有1(所有数据) 则返回所有数据
    5. 只为仅本人数据权限时只返回过滤本人数据，并且部门为自己本部门(考虑到用户会变部门，只能看当前用户所在的部门数据)
    6. 自定数据权限 获取部门，根据部门过滤
    """

    def filter_queryset(self, request, queryset, view):
        """
        接口白名单是否认证数据权限
        """
        api = request.path  # 当前请求接口
        method = request.method  # 当前请求方法
        method_list = ["GET", "POST", "PUT", "DELETE", "OPTIONS"]
        method = method_list.index(method)
        # ***接口白名单***
        api_white_list = ApiWhiteList.objects.filter(enable_datasource=True).values(
            permission__api=F("url"), permission__method=F("method")
        )
        api_white_list = [
            str(item.get("permission__api").replace("{id}", ".*?")) + ":" + str(item.get("permission__method"))
            for item in api_white_list
            if item.get("permission__api")
        ]
        for item in api_white_list:
            new_api = api + ":" + str(method)
            match_obj = re.match(item, new_api, re.M | re.I)
            if match_obj is None:
                continue
            else:
                return queryset
        """
        判断是否为超级管理员:
        如果不是超级管理员,则进入下一步权限判断
        """
        if request.user.is_superuser == 0:
            # 0. 获取用户的部门id，没有部门则返回空
            user_dept_id = getattr(request.user, "dept_id", None)
            # if not user_dept_id:
            #    return queryset.none()

            # 1. 判断过滤的数据是否有创建人所在部门 "dept_belong_id" 字段
            # if not getattr(queryset.model, "dept_belong_id", None):
            #    return queryset

            # 2. 如果用户没有关联角色则返回本部门数据
            # if not hasattr(request.user, "role"):
            #    return queryset.filter(dept_belong_id=user_dept_id)

            # 3. 根据所有角色 获取所有权限范围
            # (0, "仅本人数据权限"),
            # (1, "本部门及以下数据权限"),
            # (2, "本部门数据权限"),
            # (3, "全部数据权限"),
            # (4, "自定数据权限")
            role_list = request.user.role.filter(status=1).values("admin", "data_range")
            data_scope_list = []  # 权限范围列表
            for ele in role_list:
                # 判断用户是否为超级管理员角色/如果拥有[全部数据权限]则返回所有数据
                if 3 == ele.get("data_range") or ele.get("admin"):
                    return queryset
                data_scope_list.append(ele.get("data_range"))
            data_scope_list = list(set(data_scope_list))

            # 4. 只为仅本人数据权限时只返回过滤本人数据，并且部门为自己本部门(考虑到用户会变部门，只能看当前用户所在的部门数据)
            if 0 in data_scope_list:
                return queryset.filter(creator=request.user.id)  # dept_belong_id=user_dept_id

            # 5. 自定义数据权限 获取部门，根据部门过滤
            dept_list = []
            for ele in data_scope_list:
                if ele == 4:
                    dept_list.extend(request.user.role.filter(status=1).values_list("dept__id", flat=True))
                elif ele == 2:
                    dept_list.append(user_dept_id)
                elif ele == 1:
                    dept_list.append(user_dept_id)
                    dept_list.extend(
                        get_dept(
                            user_dept_id,
                        )
                    )
            if queryset.model._meta.model_name == "dept":
                return queryset.filter(id__in=list(set(dept_list)))
            return queryset.filter(dept_belong_id__in=list(set(dept_list)))
        else:
            return queryset


class CustomDjangoFilterBackend(DjangoFilterBackend):
    lookup_prefixes = {
        "^": "istartswith",
        "=": "iexact",
        "@": "search",
        "$": "iregex",
        "~": "icontains",
    }
    filter_fields = "__all__"

    def construct_search(self, field_name, lookup_expr=None):
        lookup = self.lookup_prefixes.get(field_name[0])
        if lookup:
            field_name = field_name[1:]
        else:
            lookup = lookup_expr
        if lookup:
            if field_name.endswith(lookup):
                return field_name
            return LOOKUP_SEP.join([field_name, lookup])
        return field_name

    def find_filter_lookups(self, orm_lookups, search_term_key):
        for lookup in orm_lookups:
            # if lookup.find(search_term_key) >= 0:
            new_lookup = LOOKUP_SEP.join(lookup.split(LOOKUP_SEP)[:-1]) if len(lookup.split(LOOKUP_SEP)) > 1 else lookup
            # 修复条件搜索错误 bug
            if new_lookup == search_term_key:
                return lookup
        return None

    def get_filterset_class(self, view, queryset=None):
        """
        Return the `FilterSet` class used to filter the queryset.
        """
        filterset_class = getattr(view, "filterset_class", None)
        filterset_fields = getattr(view, "filterset_fields", None)

        # TODO: remove assertion in 2.1
        if filterset_class is None and hasattr(view, "filter_class"):
            utils.deprecate(f"`{view.__class__.__name__}.filter_class` attribute should be renamed `filterset_class`.")
            filterset_class = getattr(view, "filter_class", None)

        # TODO: remove assertion in 2.1
        if filterset_fields is None and hasattr(view, "filter_fields"):
            utils.deprecate(
                f"`{view.__class__.__name__}.filter_fields` attribute should be renamed `filterset_fields`."
            )
            self.filter_fields = getattr(view, "filter_fields", None)
            if isinstance(self.filter_fields, (list, tuple)):
                filterset_fields = [
                    field[1:] if field[0] in self.lookup_prefixes.keys() else field for field in self.filter_fields
                ]
            else:
                filterset_fields = self.filter_fields

        if filterset_class:
            filterset_model = filterset_class._meta.model

            # FilterSets do not need to specify a Meta class
            if filterset_model and queryset is not None:
                assert issubclass(queryset.model, filterset_model), (
                    f"FilterSet model {filterset_model} does not match queryset model {queryset.model}"
                )

            return filterset_class

        if filterset_fields and queryset is not None:
            meta_base = getattr(self.filterset_base, "Meta", object)

            class AutoFilterSet(self.filterset_base):
                @classmethod
                def get_all_model_fields(cls, model):
                    opts = model._meta

                    return [
                        f.name
                        for f in sorted(opts.fields + opts.many_to_many)
                        if (f.name == "id")
                        or not isinstance(f, models.AutoField)
                        and not (getattr(f.remote_field, "parent_link", False))
                    ]

                @classmethod
                def get_fields(cls):
                    """
                    Resolve the 'fields' argument that should be used for generating filters on the
                    filterset. This is 'Meta.fields' sans the fields in 'Meta.exclude'.
                    """
                    model = cls._meta.model
                    fields = cls._meta.fields
                    exclude = cls._meta.exclude

                    assert not (fields is None and exclude is None), (
                        "Setting 'Meta.model' without either 'Meta.fields' or 'Meta.exclude' "
                        "has been deprecated since 0.15.0 and is now disallowed. Add an explicit "
                        f"'Meta.fields' or 'Meta.exclude' to the {cls.__name__} class."
                    )

                    # Setting exclude with no fields implies all other fields.
                    if exclude is not None and fields is None:
                        fields = ALL_FIELDS

                    # Resolve ALL_FIELDS into all fields for the filterset's model.
                    if fields == ALL_FIELDS:
                        fields = cls.get_all_model_fields(model)

                    # Remove excluded fields
                    exclude = exclude or []
                    if not isinstance(fields, dict):
                        fields = [(f, [settings.DEFAULT_LOOKUP_EXPR]) for f in fields if f not in exclude]
                    else:
                        fields = [(f, lookups) for f, lookups in fields.items() if f not in exclude]

                    return OrderedDict(fields)

                @classmethod
                def get_filters(cls):
                    """
                    Get all filters for the filterset. This is the combination of declared and
                    generated filters.
                    """

                    # No model specified - skip filter generation
                    if not cls._meta.model:
                        return cls.declared_filters.copy()

                    # Determine the filters that should be included on the filterset.
                    filters = OrderedDict()
                    fields = cls.get_fields()
                    undefined = []

                    for field_name, lookups in fields.items():
                        field = get_model_field(cls._meta.model, field_name)
                        from django.db import models
                        from timezone_field import TimeZoneField

                        # 不进行 过滤的model 类
                        if isinstance(field, (models.JSONField, TimeZoneField)):
                            continue
                        # warn if the field doesn't exist.
                        if field is None:
                            undefined.append(field_name)
                        # 更新默认字符串搜索为模糊搜索
                        if (
                            isinstance(field, (models.CharField))
                            and filterset_fields == "__all__"
                            and lookups == ["exact"]
                        ):
                            lookups = ["icontains"]
                        for lookup_expr in lookups:
                            filter_name = cls.get_filter_name(field_name, lookup_expr)

                            # If the filter is explicitly declared on the class, skip generation
                            if filter_name in cls.declared_filters:
                                filters[filter_name] = cls.declared_filters[filter_name]
                                continue

                            if field is not None:
                                filters[filter_name] = cls.filter_for_field(field, field_name, lookup_expr)

                    # Allow Meta.fields to contain declared filters *only* when a list/tuple
                    if isinstance(cls._meta.fields, (list, tuple)):
                        undefined = [f for f in undefined if f not in cls.declared_filters]

                    if undefined:
                        raise TypeError(
                            "'Meta.fields' must not contain non-model field names: {}".format(", ".join(undefined))
                        )

                    # Add in declared filters. This is necessary since we don't enforce adding
                    # declared filters to the 'Meta.fields' option
                    filters.update(cls.declared_filters)
                    return filters

                class Meta(meta_base):
                    model = queryset.model
                    fields = filterset_fields

            return AutoFilterSet

        return None

    def filter_queryset(self, request, queryset, view):
        filterset = self.get_filterset(request, queryset, view)
        if filterset is None:
            return queryset
        if filterset.__class__.__name__ == "AutoFilterSet":
            queryset = filterset.queryset
            filter_fields = filterset.filters if self.filter_fields == "__all__" else self.filter_fields
            orm_lookup_dict = dict(
                zip(
                    [field for field in filter_fields],
                    [filterset.filters[lookup].lookup_expr for lookup in filterset.filters.keys()],
                )
            )
            orm_lookups = [
                self.construct_search(lookup, lookup_expr) for lookup, lookup_expr in orm_lookup_dict.items()
            ]
            # print(orm_lookups)
            conditions = []
            queries = []
            for search_term_key in filterset.data.keys():
                orm_lookup = self.find_filter_lookups(orm_lookups, search_term_key)
                if not orm_lookup or filterset.data.get(search_term_key) == "":
                    continue
                filterset_data_len = len(filterset.data.getlist(search_term_key))
                if filterset_data_len == 1:
                    query = Q(**{orm_lookup: filterset.data[search_term_key]})
                    queries.append(query)
                elif filterset_data_len == 2:
                    orm_lookup += "__range"
                    query = Q(**{orm_lookup: filterset.data.getlist(search_term_key)})
                    queries.append(query)
            if len(queries) > 0:
                conditions.append(reduce(operator.and_, queries))
                queryset = queryset.filter(reduce(operator.and_, conditions))
                return queryset
            else:
                return queryset

        if not filterset.is_valid() and self.raise_exception:
            raise utils.translate_validation(filterset.errors)
        return filterset.qs


# ####################### 懒加载FilterSet ####################### #


def calculate_execution_time(func):
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        execution_time = end_time - start_time
        print(f"Function {func.__name__} took {execution_time:.6f} seconds to execute.", flush=True)
        return result

    return wrapper


# def get_children(model: models, obj_id: int, all_qs=None, rec_list=None):
#     if not all_qs:
#         all_qs = model.objects.all().values("id", "parent")
#     if rec_list is None:
#         rec_list = [obj_id]
#     for ele in all_qs:
#         if ele.get("parent") == obj_id:
#             rec_list.append(ele.get("id"))
#             get_dept(ele.get("id"), all_qs, rec_list)
#     return list(set(rec_list))


# @calculate_execution_time
# def get_qs_children(model, qs):
#     dept_ids = []
#     for d in qs:
#         r = get_children(model, d.id)
#         dept_ids.extend(r)
#     return list(set(dept_ids))


def next_layer_data(qs_filter, qs_node):
    parent_nodes = set(qs_node.values_list("id", flat=True))
    if set(qs_filter) == set(qs_node):
        return parent_nodes
    # qs_filter内所有父级id     去重
    parent_ids = set()
    for node in qs_filter:
        while node.parent:
            if node.id in parent_nodes:
                parent_ids.add(node.id)
                break
            if node.parent.id in parent_nodes:
                parent_ids.add(node.parent.id)
                break
            node = node.parent
    # print(f"过滤查询集           ==>         {qs_filter}", flush=True)
    # print(f"待渲染节点的id        ==>         {parent_nodes=}", flush=True)
    # print(f"过滤查询集的父节点id   ==>         {parent_ids=}", flush=True)
    return parent_ids


def construct_data(qs_filter, qs_node, is_parent):
    filter_node_ids = set(qs_filter.values_list("id", flat=True))
    render_node_ids = set(qs_node.values_list("id", flat=True))

    hidden_node_ids = set()
    for node in qs_filter:
        while node.parent:
            if node.parent in qs_filter:
                hidden_node_ids.add(node.id)
            node = node.parent
    on_show = filter_node_ids.difference(hidden_node_ids)
    on_expand = hidden_node_ids & render_node_ids
    # print(f"完整查询结果 {filter_node_ids}")
    # print(f"待展示的节点(未过滤) {render_node_ids}")
    # print(f"查询结果中的子节点 {hidden_node_ids}")
    # print(f"查询后首先渲染的父节点 {on_show}")
    # print(f"展开父节点时要渲染的节点 {on_expand}")
    return on_expand if is_parent else on_show


class FilterSetOptions:
    def __init__(self, options=None):
        self.model = getattr(options, "model", None)
        self.fields = getattr(options, "fields", None)
        self.exclude = getattr(options, "exclude", None)

        # CharField默认模糊查询
        self.filter_overrides = getattr(
            options,
            "filter_overrides",
            {
                models.CharField: {
                    "filter_class": CharFilter,
                    "extra": lambda f: {
                        "lookup_expr": "icontains",
                    },
                },
                models.BooleanField: {
                    "filter_class": BooleanFilter,
                    "extra": lambda f: {
                        "widget": forms.RadioSelect,
                    },
                },
            },
        )

        self.form = getattr(options, "form", forms.Form)


class LazyLoadFilterSetMetaclass(FilterSetMetaclass):
    def __new__(cls, name, bases, attrs):
        attrs["declared_filters"] = cls.get_declared_filters(bases, attrs)

        new_class = super().__new__(cls, name, bases, attrs)
        new_class._meta = FilterSetOptions(getattr(new_class, "Meta", None))
        new_class.base_filters = new_class.get_filters()

        return new_class


class LazyLoadFilter(FilterSet, metaclass=LazyLoadFilterSetMetaclass):
    @property
    # @calculate_execution_time
    def qs(self):
        queryset = self.queryset
        # print(self.form.cleaned_data, flush=True)
        filter_params = [k for k, v in self.form.cleaned_data.items() if v in [None, ""]]
        for field in filter_params:
            self.form.cleaned_data.pop(field)
        is_parent = self.form.cleaned_data.pop("parent", None) is not None
        # print(queryset, flush=True)
        if self.form.cleaned_data:
            self.queryset = queryset.model.objects.all()

            # 从根节点开始
            # node_ids = next_layer_data(super().qs, queryset)

            # 按匹配结果显示
            node_ids = construct_data(super().qs, queryset, is_parent)

            return queryset.model.objects.filter(id__in=node_ids)
        return super().qs



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# 初始化

import json
import logging
import os

from django.apps import apps

from django_base_ai.system.models import Users

logger = logging.getLogger(__name__)
from django_base_ai.system.views.api_white_list import ApiWhiteListInitSerializer
from django_base_ai.system.views.dept import DeptInitSerializer
from django_base_ai.system.views.dictionary import DictionaryInitSerializer
from django_base_ai.system.views.menu import MenuInitSerializer
from django_base_ai.system.views.role import RoleInitSerializer
from django_base_ai.system.views.system_config import SystemConfigInitSerializer
from django_base_ai.system.views.tags import TagsInitSerializer
from django_base_ai.system.views.user import UsersInitSerializer
from django_base_ai.utils.core_initialize import CoreInitialize


class Initialize(CoreInitialize):
    def init_dept(self):
        """
        初始化部门信息
        """
        self.init_base(DeptInitSerializer, unique_fields=["name", "parent", "key"])

    def init_role(self):
        """
        初始化角色信息
        """
        self.init_base(RoleInitSerializer, unique_fields=["key"])

    def _ensure_bootstrap_user(self):
        """
        在任意 init 前执行：若 init_users.json 存在且非空，则创建/更新其第一个用户并设为 request.user，
        避免整段 init 期间 request.user 为 AnonymousUser 导致误写入 AnonymousUser 记录。
        """
        path_file = os.path.join(
            apps.get_app_config(self.app.split(".")[-1]).path,
            "fixtures",
            "init_users.json",
        )
        if not os.path.isfile(path_file):
            return
        with open(path_file, encoding="utf-8") as f:
            user_list = json.load(f)
        if not user_list:
            return
        first = user_list[0]
        filter_data = {"username": first["username"]} if "username" in first else {}
        instance = Users.objects.filter(**filter_data).first()
        first["reset"] = self.reset
        serializer = UsersInitSerializer(instance, data=first, request=self.request)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        bootstrap_user = Users.objects.filter(username=first["username"]).first()
        if bootstrap_user:
            self.request.user = bootstrap_user

    def init_users(self):
        """
        初始化用户信息（run 开始时已通过 _ensure_bootstrap_user 将 request.user 设为真实用户）
        """
        self.init_base(UsersInitSerializer, unique_fields=["username"])

    def init_tags(self):
        """
        初始化标签
        """
        self.init_base(TagsInitSerializer, unique_fields=["name"])

    def init_menu(self):
        """
        初始化菜单信息
        """
        self.init_base(MenuInitSerializer, unique_fields=["name", "component"])

    def init_api_white_list(self):
        """
        初始API白名单
        """
        self.init_base(
            ApiWhiteListInitSerializer,
            unique_fields=[
                "url",
                "method",
            ],
        )

    def init_dictionary(self):
        """
        初始化字典表
        """
        self.init_base(
            DictionaryInitSerializer,
            unique_fields=[
                "value",
                "parent",
            ],
        )

    def init_system_config(self):
        """
        初始化系统配置表
        """
        self.init_base(
            SystemConfigInitSerializer,
            unique_fields=[
                "key",
                "parent",
            ],
        )

    def run(self):
        logger.info("系统初始化开始 app=%s", self.app)
        # 在任意 init 前先确保有一个真实用户并设为 request.user，避免整段 init 期间 request.user 为 AnonymousUser 导致误写库
        self._ensure_bootstrap_user()
        self.init_dept()
        self.init_role()
        self.init_users()
        self.init_tags()
        self.init_menu()
        self.init_api_white_list()
        self.init_dictionary()
        self.init_system_config()
        logger.info("系统初始化完成 app=%s", self.app)


if __name__ == "__main__":
    Initialize(app="admin.system").run()
